import datetime
import re
import json
import requests
from django.db import IntegrityError
from django.db import connection
from datetime import datetime
from gkutils.commonutils import coneSearchHTM, FULL, QUICK, COUNT, CAT_ID_RA_DEC_COLS, base26, Struct
from rest_framework import serializers
import sys
from django.core.exceptions import ObjectDoesNotExist

#from atlas.apiutils import candidateddcApi, getObjectList
#from atlas.apiutils import getVRAScoresList
#from atlas.apiutils import getVRATodoList
#from atlas.apiutils import getCustomListObjects
#from atlas.apiutils import getVRARankList

# 2024-01-29 KWS Need the model to do inserts.
#from atlas.models import TcsVraScores
#from atlas.models import AtlasDiffObjects
#from atlas.models import TcsVraTodo
#from atlas.models import TcsObjectGroups
#from atlas.models import TcsObjectGroupDefinitions
#from atlas.models import TcsVraRank

#CAT_ID_RA_DEC_COLS['objects'] = [['objectId', 'ramean', 'decmean'], 1018]

REQUEST_TYPE_CHOICES = (
    ('count', 'Count'),
    ('all', 'All'),
    ('nearest', 'Nearest'),
)


class ConeSerializer(serializers.Serializer):
    ra = serializers.FloatField(required=True)
    dec = serializers.FloatField(required=True)
    radius = serializers.FloatField(required=True)
    requestType = serializers.ChoiceField(choices=REQUEST_TYPE_CHOICES)

    def save(self):

        ra = self.validated_data['ra']
        dec = self.validated_data['dec']
        radius = self.validated_data['radius']
        requestType = self.validated_data['requestType']

        # Get the authenticated user, if it exists.
        userId = 'unknown'
        request = self.context.get("request")
        if request and hasattr(request, "user"):
            userId = request.user

        if radius > 1000:
            replyMessage = "Max radius is 1000 arcsec."
            info = {"error": replyMessage}
            return info

        replyMessage = 'No object found ra=%.5f dec=%.5f radius=%.2f' % (ra, dec, radius)
        info = {"error": replyMessage}

        # Is there an object within RADIUS arcsec of this object? - KWS - need to fix the gkhtm code!!
        # For ATLAS QUICK does not work because of the `dec` syntax error problem. Use FULL until I figure out what
        # needs to be fixed.
        message, results = coneSearchHTM(ra, dec, radius, 'tcs_transient_objects', queryType=FULL, conn=connection, django=True)

        obj = None
        separation = None

        objectList = []
        if len(results) > 0:
            if requestType == "nearest":
                obj = results[0][1]['id']
                separation = results[0][0]
                info = {"object": obj, "separation": separation}
            elif requestType == "all":
                for row in results:
                    objectList.append({"object": row[1]["id"], "separation": row[0]})
                info = objectList
            elif requestType == "count":
                info = {'count': len(results)}
            else:
                info = {"error": "Invalid request type"}

        return info


#class ObjectsSerializer(serializers.Serializer):
#    objects = serializers.CharField(required=True)
#    mjd = serializers.FloatField(required=False, default=None)
#
#    def save(self):
#        objects = self.validated_data['objects']
#        mjd = self.validated_data['mjd']
#
#        olist = []
#        for tok in objects.split(','):
#            olist.append(tok.strip())
#
#        if len(olist) > 100:
#            return {"info": "Max number of objects for each requests is 100"}
##        olist = olist[:10] # restrict to 10
#
#        # Get the authenticated user, if it exists.
#        userId = 'unknown'
#        request = self.context.get("request")
#        if request and hasattr(request, "user"):
#            userId = request.user
#
#
#        result = []
#        for candidate in olist:
#            result.append(candidateddcApi(request, candidate, mjdThreshold=mjd))
#        return result
#
#
#class ObjectListSerializer(serializers.Serializer):
#    objectlistid = serializers.IntegerField(required=True)
#    getcustomlist = serializers.BooleanField(required=False, default = False)
#    datethreshold = serializers.DateTimeField(required=False, default=None)
#
#    def save(self):
#        objectlistid = self.validated_data['objectlistid']
#        getcustomlist = self.validated_data['getcustomlist']
#        datethreshold = self.validated_data['datethreshold']
#
#        request = self.context.get("request")
#
#        dateThreshold = None
#        if datethreshold is not None:
#            dateThreshold = self.validated_data['datethreshold']
#
#        objectList = getObjectList(request, objectlistid, getCustomList = getcustomlist, dateThreshold = dateThreshold)
#        return objectList
#
#
## 2024-01-17 KWS Insert a VRA row for an object. For the time being do this one at a time.
## 2024-02-21 KWS Changed required to False for all three prob values.
#class VRAScoresSerializer(serializers.Serializer):
#    objectid = serializers.IntegerField(required=True)
#    preal = serializers.FloatField(required=False, default=None)
#    pgal = serializers.FloatField(required=False, default=None)
#    pfast = serializers.FloatField(required=False, default=None)
#    # 2024-08-14 KWS Added 3 new values to the form for Rank.
#    rank = serializers.FloatField(required=True)
#    rank_alt1 = serializers.FloatField(required=False, default=None)
#    rank_alt2 = serializers.FloatField(required=False, default=None)
#    debug = serializers.BooleanField(required=False, default=False)
#    insertdate = serializers.DateTimeField(required=False, default=None)
#
#
#    def save(self):
#
#        from django.conf import settings
#        objectid = self.validated_data['objectid']
#        preal = self.validated_data['preal']
#        pgal = self.validated_data['pgal']
#        pfast = self.validated_data['pfast']
#        rank = self.validated_data['rank']
#        rank_alt1 = self.validated_data['rank_alt1']
#        rank_alt2 = self.validated_data['rank_alt2']
#        insertdate = self.validated_data['insertdate']
#        debug = self.validated_data['debug']
#
#        insertDate = None
#        if insertdate is not None:
#            insertDate = self.validated_data['insertdate']
#
#        replyMessage = 'Row created.'
#
#        userId = 'unknown'
#        request = self.context.get("request")
#        if request and hasattr(request, "user"):
#            userId = str(request.user)
#
#        if not insertDate:
#            insertDate = datetime.now()
#
#        data = {'transient_object_id_id': objectid,
#                'preal': preal,
#                'pgal': pgal,
#                'pfast': pfast,
#                'rank': rank,
#                'rank_alt1': rank_alt1,
#                'rank_alt2': rank_alt2,
#                'timestamp': insertDate,
#                'debug': debug,
#                'apiusername': userId}
#
#        # Does the objectId actually exit - not allowed to comment on objects that don't exist!
#        # This should really return a 404 message.
#        try:
#            transient = AtlasDiffObjects.objects.get(pk=objectid)
#        except ObjectDoesNotExist as e:
#            replyMessage = 'Object does not exist.'
#            info = { "objectid": objectid, "info": replyMessage }
#            return info
#
#        ## Does the VRA row exist?
#        #vra = None
#        #try:
#        #    vra = TcsVraScores.objects.get(transient_object_id_id=objectid, debug=debug)
#        
#        #except ObjectDoesNotExist as e:
#        #    # That's OK - we'll create a new object
#        #    pass
#        
#        #if vra:
#        #    instance = vra
#        #else:
#        instance = TcsVraScores(**data)
#        #try:
#
#        #    if vra is not None:
#        #        instance.preal = preal
#        #        instance.pfast = pfast
#        #        instance.pgal = pgal
#        #        instance.timestamp = insertDate
#        #        i = instance.save()
#        #    else:
#        i = instance.save()
#
#        #except IntegrityError as e:
#        #    replyMessage = 'Duplicate row. Cannot add row.'
#
#        info = { "objectid": objectid, "info": replyMessage }
#        return info
#
#
#
#class VRAScoresListSerializer(serializers.Serializer):
#    objects = serializers.CharField(required=False, default=None)
#    debug = serializers.BooleanField(required=False, default=False)
#    datethreshold = serializers.DateTimeField(required=False, default='1970-01-01')
#    idthreshold = serializers.IntegerField(required=False, default=0)
#
#    def save(self):
#        objects = self.validated_data['objects']
#        datethreshold = self.validated_data['datethreshold']
#        debug = self.validated_data['debug']
#        idthreshold = self.validated_data['idthreshold']
#
#        request = self.context.get("request")
#
#        olist = []
#
#        if objects is not None:
#            for tok in objects.split(','):
#                olist.append(tok.strip())
#
#        vraScoresList = getVRAScoresList(request, objects = olist, debug = debug, dateThreshold = datethreshold, idThreshold = idthreshold)
#        return vraScoresList
#
#
## 2024-02-21 KWS Changed required to False for all three prob values.
#class VRATodoSerializer(serializers.Serializer):
#    objectid = serializers.IntegerField(required=True)
#    insertdate = serializers.DateTimeField(required=False, default=None)
#
#    import sys
#
#    def save(self):
#
#        from django.conf import settings
#        objectid = self.validated_data['objectid']
#        insertdate = self.validated_data['insertdate']
#
#        insertDate = None
#        if insertdate is not None:
#            insertDate = self.validated_data['insertdate']
#
#        replyMessage = 'Row created.'
#
#        if not insertDate:
#            insertDate = datetime.now()
#
#        data = {'transient_object_id_id': objectid,
#                'timestamp': insertDate}
#
#        # Does the objectId actually exit - not allowed to comment on objects that don't exist!
#        # This should really return a 404 message.
#        try:
#            transient = AtlasDiffObjects.objects.get(pk=objectid)
#        except ObjectDoesNotExist as e:
#            replyMessage = 'Object does not exist.'
#            info = { "objectid": objectid, "info": replyMessage }
#            return info
#
#        try:
#            instance = TcsVraTodo(**data)
#            i = instance.save(force_insert=True)
#            # NOTE: Inserting an object by setting the primary key actually REPLACES the object. Do we want this behaviour??
#            #       The integrity error below never happens because I've now set the model with primary_key=True.
#            #       To fix this I've added force_insert = True above.
#        except IntegrityError as e:
#            replyMessage = 'Duplicate row. Cannot add row.'
#
#        info = { "objectid": objectid, "info": replyMessage }
#        return info
#
#
#class VRATodoListSerializer(serializers.Serializer):
#    objects = serializers.CharField(required=False, default=None)
#    datethreshold = serializers.DateTimeField(required=False, default='1970-01-01')
#    idthreshold = serializers.IntegerField(required=False, default=0)
#
#    def save(self):
#        objects = self.validated_data['objects']
#        datethreshold = self.validated_data['datethreshold']
#        idthreshold = self.validated_data['idthreshold']
#
#        request = self.context.get("request")
#
#        olist = []
#
#        if objects is not None:
#            for tok in objects.split(','):
#                olist.append(tok.strip())
#
#        vraTodoList = getVRATodoList(request, objects = olist, dateThreshold = datethreshold, idThreshold = idthreshold)
#        return vraTodoList
#
#
## 2024-02-21 KWS Changed required to False for all three prob values.
#class TcsObjectGroupsSerializer(serializers.Serializer):
#    objectid = serializers.IntegerField(required=True)
#    objectgroupid = serializers.IntegerField(required=True)
#
#    import sys
#
#    def save(self):
#
#        from django.conf import settings
#        objectid = self.validated_data['objectid']
#        objectGroupId = self.validated_data['objectgroupid']
#
#        replyMessage = 'Row created.'
#
#        # This is what gets inserted into the database.
#        data = {'transient_object_id_id': objectid,
#                'object_group_id': objectGroupId}
#
#        # Does the objectId actually exit - not allowed to comment on objects that don't exist!
#        # This should really return a 404 message.
#        try:
#            transient = AtlasDiffObjects.objects.get(pk=objectid)
#        except ObjectDoesNotExist as e:
#            replyMessage = 'Object does not exist.'
#            info = { "objectid": objectid, "info": replyMessage }
#            return info
#
#        try:
#            group = TcsObjectGroupDefinitions.objects.get(pk=objectGroupId)
#        except ObjectDoesNotExist as e:
#            replyMessage = 'Object group ID does not exist.'
#            info = { "objectgroupid": objectGroupId, "info": replyMessage }
#            return info
#
#        try:
#            instance = TcsObjectGroups(**data)
#            i = instance.save(force_insert=True)
#            # NOTE: Inserting an object by setting the primary key actually REPLACES the object. Do we want this behaviour??
#            #       The integrity error below never happens because I've now set the model with primary_key=True.
#            #       To fix this I've added force_insert = True above.
#        except IntegrityError as e:
#            replyMessage = 'Duplicate row. Cannot add row.'
#
#        info = { "objectgroupid": objectid, "info": replyMessage }
#        return info
#
## 2024-02-21 KWS Changed required to False for all three prob values.
#class TcsObjectGroupsDeleteSerializer(serializers.Serializer):
#    objectid = serializers.IntegerField(required=True)
#    objectgroupid = serializers.IntegerField(required=True)
#
#    import sys
#
#    def save(self):
#
#        from django.conf import settings
#        objectid = self.validated_data['objectid']
#        objectGroupId = self.validated_data['objectgroupid']
#
#        replyMessage = 'Row deleted.'
#
#        # This is what gets inserted into the database.
#        data = {'transient_object_id_id': objectid,
#                'object_group_id': objectGroupId}
#
#        # Does the objectId actually exit - not allowed to comment on objects that don't exist!
#        # This should really return a 404 message.
#        try:
#            transient = AtlasDiffObjects.objects.get(pk=objectid)
#        except ObjectDoesNotExist as e:
#            replyMessage = 'Object does not exist.'
#            info = { "objectid": objectid, "info": replyMessage }
#            return info
#
#        try:
#            group = TcsObjectGroupDefinitions.objects.get(pk=objectGroupId)
#        except ObjectDoesNotExist as e:
#            replyMessage = 'Object group ID does not exist.'
#            info = { "objectgroupid": objectGroupId, "info": replyMessage }
#            return info
#
#        try:
#            instance = TcsObjectGroups.objects.get(transient_object_id__id = objectid, object_group_id = objectGroupId)
#        except ObjectDoesNotExist as e:
#            replyMessage = 'Object group ID does not exist or object ID does not exist.'
#            info = { "objectgroupid": objectGroupId, "objectid": objectid, "info": replyMessage }
#            return info
#
#        i = instance.delete()
#        #replyMessage = 'Duplicate row. Cannot add row.'
#
#        info = { "objectgroupid": objectid, "info": replyMessage }
#        return info
#
#class TcsObjectGroupsListSerializer(serializers.Serializer):
#    objectid = serializers.IntegerField(required=False, default=None)
#    objectgroupid = serializers.IntegerField(required=False, default=None)
#
#    def save(self):
#        objectid = self.validated_data['objectid']
#        objectGroupId = self.validated_data['objectgroupid']
#
#        request = self.context.get("request")
#
#        customListObjects = getCustomListObjects(request, objectid, objectGroupId)
#        return customListObjects
#
#
## 2024-05-22 KWS Added VRARank and VRARankList Serializers.
#class VRARankSerializer(serializers.Serializer):
#    objectid = serializers.IntegerField(required=True)
#    rank = serializers.FloatField(required=True)
#    rank_alt1 = serializers.FloatField(required=False, default=None)
#    rank_alt2= serializers.FloatField(required=False, default=None)
#    insertdate = serializers.DateTimeField(required=False, default=None)
#
#    import sys
#
#    def save(self):
#
#        from django.conf import settings
#        objectid = self.validated_data['objectid']
#        insertdate = self.validated_data['insertdate']
#        rank = self.validated_data['rank']
#        rank_alt1 = self.validated_data['rank_alt1']
#        rank_alt2 = self.validated_data['rank_alt2']
#
#        insertDate = None
#        if insertdate is not None:
#            insertDate = self.validated_data['insertdate']
#
#        replyMessage = 'Row created.'
#
#        if not insertDate:
#            insertDate = datetime.now()
#
#        data = {'transient_object_id_id': objectid,
#                'rank': rank,
#                'rank_alt1': rank_alt1,
#                'rank_alt2': rank_alt2,
#                'timestamp': insertDate}
#
#        # Does the objectId actually exit - not allowed to comment on objects that don't exist!
#        # This should really return a 404 message.
#        try:
#            transient = AtlasDiffObjects.objects.get(pk=objectid)
#        except ObjectDoesNotExist as e:
#            replyMessage = 'Object does not exist.'
#            info = { "objectid": objectid, "info": replyMessage }
#            return info
#
#        try:
#            instance = TcsVraRank(**data)
#            # Do not use force_insert=True here. We really want to update any existing row
#            # or insert if the row doesn't exist.
#            i = instance.save()
#
#        except IntegrityError as e:
#            # Should never happen!
#            sys.stderr.write(str(e))
#            replyMessage = 'Duplicate row. Cannot add row.'
#
#        info = { "objectid": objectid, "info": replyMessage }
#        return info
#
#
#class VRARankListSerializer(serializers.Serializer):
#    objects = serializers.CharField(required=False, default=None)
#    datethreshold = serializers.DateTimeField(required=False, default='1970-01-01')
#
#    def save(self):
#        objects = self.validated_data['objects']
#        datethreshold = self.validated_data['datethreshold']
#
#        request = self.context.get("request")
#
#        olist = []
#
#        if objects is not None:
#            for tok in objects.split(','):
#                olist.append(tok.strip())
#
#        vraRankList = getVRARankList(request, objects = olist, dateThreshold = datethreshold)
#        return vraRankList
#
#



from django.shortcuts import get_object_or_404, render
from rest_framework.response import Response
from rest_framework.views import APIView
from rest_framework import status
#from .serializers import ConeSerializer, ObjectsSerializer, ObjectListSerializer, VRAScoresSerializer, VRAScoresListSerializer, VRATodoSerializer, VRATodoListSerializer, TcsObjectGroupsSerializer, TcsObjectGroupsDeleteSerializer, TcsObjectGroupsListSerializer, VRARankSerializer, VRARankListSerializer
from .serializers import ConeSerializer
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
from .query_auth import QueryAuthentication
from django.core.exceptions import ObjectDoesNotExist
# 2024-01-29 KWS Need the model to do inserts.
#from atlas.models import TcsVraScores
#from atlas.models import TcsObjectGroups
import sys

def retcode(message):
    if 'error' in message: return status.HTTP_400_BAD_REQUEST
    else:                  return status.HTTP_200_OK

class ConeView(APIView):
    authentication_classes = [TokenAuthentication, QueryAuthentication]
    permission_classes = [IsAuthenticated]

    def get(self, request):
        serializer = ConeSerializer(data=request.GET, context={'request': request})
        if serializer.is_valid():
            message = serializer.save()
            return Response(message, status=retcode(message))
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    def post(self, request, format=None):
        serializer = ConeSerializer(data=request.data, context={'request': request})
        if serializer.is_valid():
            message = serializer.save()
            return Response(message, status=retcode(message))
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


#class ObjectsView(APIView):
#    authentication_classes = [TokenAuthentication, QueryAuthentication]
#    permission_classes = [IsAuthenticated]
#
#    def get(self, request):
#        serializer = ObjectsSerializer(data=request.GET, context={'request': request})
#        if serializer.is_valid():
#            message = serializer.save()
#            return Response(message, status=retcode(message))
#        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
#
#    def post(self, request, format=None):
#        serializer = ObjectsSerializer(data=request.data, context={'request': request})
#        if serializer.is_valid():
#            message = serializer.save()
#            return Response(message, status=retcode(message))
#        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
#
#
#class ObjectListView(APIView):
#    authentication_classes = [TokenAuthentication, QueryAuthentication]
#    permission_classes = [IsAuthenticated]
#
#    def get(self, request):
#        serializer = ObjectListSerializer(data=request.GET, context={'request': request})
#        if serializer.is_valid():
#            message = serializer.save()
#            return Response(message, status=retcode(message))
#        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
#
#    def post(self, request, format=None):
#        serializer = ObjectListSerializer(data=request.data, context={'request': request})
#        if serializer.is_valid():
#            message = serializer.save()
#            return Response(message, status=retcode(message))
#        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
#
#
#class VRAScoresView(APIView):
#    authentication_classes = [TokenAuthentication, QueryAuthentication]
#    permission_classes = [IsAuthenticated]
#
#    def get(self, request):
#        return Response({"Error": "GET is not implemented for this service."})
#
#    def post(self, request, format=None):
#        serializer = VRAScoresSerializer(data=request.data, context={'request': request})
#
#        if serializer.is_valid():
#            message = serializer.save()
#
#            return Response(message, status=status.HTTP_201_CREATED)
#        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
#
#
#class VRAScoresListView(APIView):
#    authentication_classes = [TokenAuthentication, QueryAuthentication]
#    permission_classes = [IsAuthenticated]
#
#    def get(self, request):
#        serializer = VRAScoresListSerializer(data=request.GET, context={'request': request})
#        if serializer.is_valid():
#            message = serializer.save()
#            return Response(message, status=retcode(message))
#        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
#
#    def post(self, request, format=None):
#        serializer = VRAScoresListSerializer(data=request.data, context={'request': request})
#        if serializer.is_valid():
#            message = serializer.save()
#            return Response(message, status=retcode(message))
#        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
#
## 2024-04-16 KWS Added VRATodoView. NOTE: Add code to read the reply message and generate a sensible HTTP response
##                appropriate to the circumstances. E.g. if object is not found generate a 404, etc.
#class VRATodoView(APIView):
#    authentication_classes = [TokenAuthentication, QueryAuthentication]
#    permission_classes = [IsAuthenticated]
#
#    def get(self, request):
#        return Response({"Error": "GET is not implemented for this service."})
#
#    def post(self, request, format=None):
#        serializer = VRATodoSerializer(data=request.data, context={'request': request})
#    
#        if serializer.is_valid():
#            message = serializer.save()
#    
#            return Response(message, status=status.HTTP_201_CREATED)
#        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
#            
## 2024-05-07 KWS Added VRATodoListView.
#class VRATodoListView(APIView):
#    authentication_classes = [TokenAuthentication, QueryAuthentication]
#    permission_classes = [IsAuthenticated]
#
#    def get(self, request):
#        serializer = VRATodoListSerializer(data=request.GET, context={'request': request})
#        if serializer.is_valid():
#            message = serializer.save()
#            return Response(message, status=retcode(message))
#        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
#
#    def post(self, request, format=None):
#        serializer = VRATodoListSerializer(data=request.data, context={'request': request})
#        if serializer.is_valid():
#            message = serializer.save()
#            return Response(message, status=retcode(message))
#        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
#
#class TcsObjectGroupsView(APIView):
#    authentication_classes = [TokenAuthentication, QueryAuthentication]
#    permission_classes = [IsAuthenticated]
#
#    def get(self, request):
#        return Response({"Error": "GET is not implemented for this service."})
#
#    def post(self, request, format=None):
#        serializer = TcsObjectGroupsSerializer(data=request.data, context={'request': request})
#    
#        if serializer.is_valid():
#            message = serializer.save()
#    
#            return Response(message, status=status.HTTP_201_CREATED)
#        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
#
#class TcsObjectGroupsListView(APIView):
#    authentication_classes = [TokenAuthentication, QueryAuthentication]
#    permission_classes = [IsAuthenticated]
#
#    def get(self, request):
#        serializer = TcsObjectGroupsListSerializer(data=request.GET, context={'request': request})
#        if serializer.is_valid():
#            message = serializer.save()
#            return Response(message, status=retcode(message))
#        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
#
#    def post(self, request, format=None):
#        serializer = TcsObjectGroupsListSerializer(data=request.data, context={'request': request})
#        if serializer.is_valid():
#            message = serializer.save()
#            return Response(message, status=retcode(message))
#        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
#
#
#class TcsObjectGroupsDeleteView(APIView):
#    authentication_classes = [TokenAuthentication, QueryAuthentication]
#    permission_classes = [IsAuthenticated]
#
#    def get(self, request):
#        return Response({"Error": "GET is not implemented for this service."})
#
#    def post(self, request, format=None):
#        serializer = TcsObjectGroupsDeleteSerializer(data=request.data, context={'request': request})
#    
#        if serializer.is_valid():
#            message = serializer.save()
#    
#        if "deleted" in message['info']:
#            # No point returning the message info. 204s will drop the message content anyway.
#            return Response(status=status.HTTP_204_NO_CONTENT)
#        elif "does not exist" in message['info']:
#            return Response(message, status=status.HTTP_400_BAD_REQUEST)
#        else:
#            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
#
## 2024-05-22 KWS Added VRARankView. NOTE: Add code to read the reply message and generate a sensible HTTP response
##                appropriate to the circumstances. E.g. if object is not found generate a 404, etc.
#class VRARankView(APIView):
#    authentication_classes = [TokenAuthentication, QueryAuthentication]
#    permission_classes = [IsAuthenticated]
#
#    def get(self, request):
#        return Response({"Error": "GET is not implemented for this service."})
#
#    def post(self, request, format=None):
#        serializer = VRARankSerializer(data=request.data, context={'request': request})
#    
#        if serializer.is_valid():
#            message = serializer.save()
#    
#            return Response(message, status=status.HTTP_201_CREATED)
#        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
#            
## 2024-05-22 KWS Added VRARankListView.
#class VRARankListView(APIView):
#    authentication_classes = [TokenAuthentication, QueryAuthentication]
#    permission_classes = [IsAuthenticated]
#
#    def get(self, request):
#        serializer = VRARankListSerializer(data=request.GET, context={'request': request})
#        if serializer.is_valid():
#            message = serializer.save()
#            return Response(message, status=retcode(message))
#        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
#
#    def post(self, request, format=None):
#        serializer = VRARankListSerializer(data=request.data, context={'request': request})
#        if serializer.is_valid():
#            message = serializer.save()
#            return Response(message, status=retcode(message))
#        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
#



from django.urls import include, path
from rest_framework import routers
from rest_framework.authtoken.views import obtain_auth_token
from . import views

urlpatterns = [
    path('api/cone/',                  views.ConeView.as_view()),
#    path('api/objects/',               views.ObjectsView.as_view()),
#    path('api/objectlist/',            views.ObjectListView.as_view()),
#    path('api/vrascores/',             views.VRAScoresView.as_view()),
#    path('api/vrascoreslist/',         views.VRAScoresListView.as_view()),
#    path('api/vratodo/',               views.VRATodoView.as_view()),
#    path('api/vratodolist/',           views.VRATodoListView.as_view()),
#    path('api/objectgroups/',          views.TcsObjectGroupsView.as_view()),
#    path('api/objectgroupslist/',      views.TcsObjectGroupsListView.as_view()),
#    path('api/objectgroupsdelete/',    views.TcsObjectGroupsDeleteView.as_view()),
#    path('api/vrarank/',               views.VRARankView.as_view()),
#    path('api/vraranklist/',           views.VRARankListView.as_view()),
    path('api/auth-token/',            obtain_auth_token, name='auth_token'),
]



# Functions within this file are too Django-specific to include in 'utils'.

# PS1 version of the Django Search Form Helper Function.

from psdb.models import TcsTransientObjects
from psdb.models import TcsImages, TcsLatestObjectStats
from psdb.models import TcsTransientReobservations
from psdb.lightcurvequeries import getRecurrenceDataForPlotting
from gkutils.commonutils import getObjectNamePortion, getCoordsAndSearchRadius, coneSearchHTM, FULL, Struct, ra_to_sex, dec_to_sex, CAT_ID_RA_DEC_COLS
from django.db.models import Q    # Need Q objects for OR query
import sys

SHOW_LC_DATA_LIMIT = 50


def processSearchForm(searchText, getAssociatedData = False, getNonDets = False, getNearbyObjects = False):
    """processSearchForm.

    Args:
        searchText:
        getAssociatedData:
        getNonDets:
        getNearbyObjects:
    """
    from django.db import connection

    results = []
    # Is it a name?
    name = getObjectNamePortion(searchText)
    if name:
        #name = name.replace(' ','')
        # Do a name search
        if name.startswith('20'):
            results = TcsTransientObjects.objects.filter(Q(other_designation__isnull = False) & Q(other_designation__startswith = name))
            # No results? Try the external crossmatches table.
            if len(results) == 0:
                q = TcsLatestObjectStats.objects.filter(external_crossmatches__isnull = False).filter(external_crossmatches__contains=name)
                results = list(set([x.id for x in q]))
        elif name.startswith('PS'):
            results = TcsTransientObjects.objects.filter(Q(ps1_designation__isnull = False) & Q(ps1_designation__startswith = name))
            # No results? Try the external crossmatches table.
            if len(results) == 0:
                q = TcsLatestObjectStats.objects.filter(external_crossmatches__isnull = False).filter(external_crossmatches__contains=name)
                results = list(set([x.id for x in q]))
        elif name.startswith('AT'):
            results = TcsTransientObjects.objects.filter(Q(other_designation__isnull = False) & Q(other_designation__startswith = name.replace('AT','')))
            # No results? Try the external crossmatches table.
            if len(results) == 0:
                q = TcsLatestObjectStats.objects.filter(external_crossmatches__isnull = False).filter(external_crossmatches__contains=name)
                results = list(set([x.id for x in q]))
        elif name.startswith('SN'):
            results = TcsTransientObjects.objects.filter(Q(other_designation__isnull = False) & Q(other_designation__startswith = name.replace('SN','')))
            # No results? Try the external crossmatches table.
            if len(results) == 0:
                q = TcsLatestObjectStats.objects.filter(external_crossmatches__isnull = False).filter(external_crossmatches__contains=name)
                results = list(set([x.id for x in q]))
        else: # name.startswith('ZTF'):
            # Check for external crossmatches.
            q = TcsLatestObjectStats.objects.filter(external_crossmatches__isnull = False).filter(external_crossmatches__contains=name)
            results = list(set([x.id for x in q]))
    else:
         # It must be a coordinate or a match failure
         coords = getCoordsAndSearchRadius(searchText)
         if coords:
             searchRadius = 4.0
             if coords['radius']:
                 searchRadius = float(coords['radius'])
                 if searchRadius > 99.0:
                      searchRadius = 99.0

             message, xmObjects = coneSearchHTM(coords['ra'], coords['dec'], searchRadius, 'tcs_transient_objects', queryType = FULL, conn = connection, django = True)
             for xm in xmObjects:
                 dictRow = xm[1]
                 dictRow.update({'xmseparation': xm[0]})
                 # Add sexagesimal RA and Dec
                 dictRow.update({'ra_sex': ra_to_sex(dictRow['ra_psf'])})
                 dictRow.update({'dec_sex': dec_to_sex(dictRow['dec_psf'])})
                 if dictRow['tcs_images_id']:
                     # Get images
                     images = TcsImages.objects.get(pk=dictRow['tcs_images_id'])
                     if images:
                         dictRow.update({'tcs_images_id': images})
                 # 2018-08-07 KWS Convert the class into a dict so we can use common code elsewhere.
                 results.append(Struct(**dictRow))

    return results


def getNearbyObjectsForScatterPlot(candidate, ra, dec, coneSearchRadius = 4.0):
    """getNearbyObjectsForScatterPlot.

    Args:
        candidate:
        ra:
        dec:
        coneSearchRadius:
    """
    from django.db import connection

    recurrencePlotData = []
    recurrencePlotLabels = []
    averageObjectCoords = []
    rmsScatter = []

    xmObjects = None

    # Grab all objects within 3 arcsec of this one.
    xmList = []
    catalogueName = 'tcs_transient_objects'
    message, xmObjects = coneSearchHTM(ra, dec, coneSearchRadius, catalogueName, queryType = FULL, conn = connection, django = True)

    # The crossmatch Objects xmObjects are a list of two entry lists. The first entry in each row is the separaion.
    # The second entry is the catalogue row dictionary listing the relevant crossmatch.

    if xmObjects:
        numberOfMatches = len(xmObjects)
        # Add the objects into a list of dicts that have consistent names for all catalogues

        for xm in xmObjects:
            sys.stderr.write("\n%s\n" % str(xm))
            # Add to the list all object ids except the current one (which will have
            # a separation of zero).
            if xm[1][CAT_ID_RA_DEC_COLS[catalogueName][0][0]] != candidate:
                xmList.append({'xmseparation': xm[0], 'xmid': xm[1][CAT_ID_RA_DEC_COLS[catalogueName][0][0]]})
                xmid = xm[1][CAT_ID_RA_DEC_COLS[catalogueName][0][0]]
                xmra = xm[1][CAT_ID_RA_DEC_COLS[catalogueName][0][1]]
                xmdec = xm[1][CAT_ID_RA_DEC_COLS[catalogueName][0][2]]
                xmName = xm[1]["ps1_designation"]
                if not xmName:
                    xmName = xm[1]["local_designation"]
                if not xmName:
                    xmName = xmid

                xmRecurrences = []
                xmDetections = TcsTransientReobservations.objects.filter(transient_object_id = xmid)
                for xmDet in xmDetections:
                    xmRecurrences.append({"RA": xmDet.ra_psf, "DEC": xmDet.dec_psf})
                # Add the recurrence from the objects table!
                xmRecurrences.append({"RA": xmra, "DEC": xmdec})

                xmrecurrencePlotData, xmrecurrencePlotLabels, xmaverageObjectCoords, xmrmsScatter = getRecurrenceDataForPlotting(candidate, ra, dec, secRA = xmra, secDEC = xmdec, secId = xmid, secName = xmName, objectColour = 23)
                recurrencePlotData += xmrecurrencePlotData
                recurrencePlotLabels += xmrecurrencePlotLabels
                averageObjectCoords += xmaverageObjectCoords
                rmsScatter += xmrmsScatter

    recurrenceData = [recurrencePlotData, recurrencePlotLabels, averageObjectCoords, rmsScatter]
    return recurrenceData



















import socket
# 2019-08-07 KWS added sendMessage to send a message to a listening daemon. This might be for:
#                * getting a PS name
#                * registering on TNS
#                * doing a minor planet check.

def sendMessage(host, port, message):
    '''
    Send a message to a listener.
    '''

    response = None

    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.connect((host, port))
        s.sendall(message.encode('utf-8'))
        response = s.recv(1024).decode('utf-8')
        s.close()
    except socket.error as e:
        sys.stderr.write("Cannot connect. Maybe the service has not been started or is listening on a different port.\n")
        sys.stderr.write("%s" % str(e))

    return response

def filterGetParameters(request, queryFilter, prefix = ''):
    """filterGetParameters.

    Args:
        request:
        queryFilter:
        prefix:
    """
    # When we're querying for tcs_transient_objects we probably have lots of
    # common things to pull out.  Hence this helper function. Prefix can
    # be used if we need to refer the parameters remotely, but note that it
    # applies to all parameters at the moment.

    # 2020-10-13 KWS Add ability to do greater than, greater than or equal to,
    #                less than, less than or equal to.
    import datetime
    flagDate = None
    flagParameter = 'followup_flag_date'
    for suffix in ['__lt', '__lte', '__gt', '__gte', '']:
        flagParameter = 'followup_flag_date' + suffix
        flagDate = request.GET.get(flagParameter)
        if flagDate is not None:
            break

    try:
        flagDate = datetime.datetime.strptime(flagDate, "%Y-%m-%d").date()
    except ValueError as e:
        flagDate = None
    except TypeError as e:
        flagDate = None

    if flagDate:
        queryFilter[prefix + flagParameter] = flagDate

    # 2020-10-13 KWS Added rb_pix (i.e. RB factor)
    zooniverseScore = None
    zooniverseScoreParameter = 'zooniverse_score'
    for suffix in ['__lt', '__lte', '__gt', '__gte', '']:
        zooniverseScoreParameter = 'zooniverse_score' + suffix
        zooniverseScore = request.GET.get(zooniverseScoreParameter)
        if zooniverseScore is not None:
            break

    try:
        zooniverseScore = float(zooniverseScore)
    except ValueError as e:
        zooniverseScore = None
    except TypeError as e:
        zooniverseScore = None

    if zooniverseScore:
        queryFilter[prefix + zooniverseScoreParameter] = zooniverseScore

    # 2020-10-13 KWS Added rb_pix (i.e. RB factor)
    rbPix = None
    rbPixParameter = 'rb_pix'
    for suffix in ['__lt', '__lte', '__gt', '__gte', '']:
        rbPixParameter = 'rb_pix' + suffix
        rbPix = request.GET.get(rbPixParameter)
        if rbPix is not None:
            break

    try:
        rbPix = float(rbPix)
    except ValueError as e:
        rbPix = None
    except TypeError as e:
        rbPix = None

    if rbPix:
        queryFilter[rbPixParameter] = rbPix

    # 2020-12-11 KWS Added rb_cat (i.e. catalogue RB factor)
    rbCat = None
    rbCatParameter = 'rb_cat'
    for suffix in ['__lt', '__lte', '__gt', '__gte', '']:
        rbCatParameter = 'rb_cat' + suffix
        rbCat = request.GET.get(rbCatParameter)
        if rbCat is not None:
            break

    try:
        rbCat = float(rbCat)
    except ValueError as e:
        rbCat = None
    except TypeError as e:
        rbCat = None

    if rbCat:
        queryFilter[rbCatParameter] = rbCat

    # 2020-12-11 KWS Added earliest mag & latest mag
    latestMag = None
    latestMagParameter = 'latest_mag'
    for suffix in ['__lt', '__lte', '__gt', '__gte', '']:
        latestMagParameter = 'latest_mag' + suffix
        latestMag = request.GET.get(latestMagParameter)
        if latestMag is not None:
            break

    try:
        latestMag = float(latestMag)
    except ValueError as e:
        latestMag = None
    except TypeError as e:
        latestMag = None

    if latestMag:
        queryFilter[latestMagParameter] = latestMag


    # 2020-12-11 KWS Added earliest mag & earliest mag
    earliestMag = None
    earliestMagParameter = 'earliest_mag'
    for suffix in ['__lt', '__lte', '__gt', '__gte', '']:
        earliestMagParameter = 'earliest_mag' + suffix
        earliestMag = request.GET.get(earliestMagParameter)
        if earliestMag is not None:
            break

    try:
        earliestMag = float(earliestMag)
    except ValueError as e:
        earliestMag = None
    except TypeError as e:
        earliestMag = None

    if earliestMag:
        queryFilter[earliestMagParameter] = earliestMag


    # 2020-12-11 KWS Added earliest mag & earliest mag
    earliestMJD = None
    earliestMJDParameter = 'earliest_mjd'
    for suffix in ['__lt', '__lte', '__gt', '__gte', '']:
        earliestMJDParameter = 'earliest_mjd' + suffix
        earliestMJD = request.GET.get(earliestMJDParameter)
        if earliestMJD is not None:
            break

    try:
        earliestMJD = float(earliestMJD)
    except ValueError as e:
        earliestMJD = None
    except TypeError as e:
        earliestMJD = None

    if earliestMJD:
        queryFilter[earliestMJDParameter] = earliestMJD


    # 2020-12-11 KWS Added latest mag & latest mag
    latestMJD = None
    latestMJDParameter = 'latest_mjd'
    for suffix in ['__lt', '__lte', '__gt', '__gte', '']:
        latestMJDParameter = 'latest_mjd' + suffix
        latestMJD = request.GET.get(latestMJDParameter)
        if latestMJD is not None:
            break

    try:
        latestMJD = float(latestMJD)
    except ValueError as e:
        latestMJD = None
    except TypeError as e:
        latestMJD = None

    if latestMJD:
        queryFilter[latestMJDParameter] = latestMJD


    # 2020-12-11 KWS Added earliest mag & earliest mag
    otherDesignation = None
    otherDesignationParameter = 'other_designation'
    for suffix in ['__startswith', '__endswith', '']:
        otherDesignationParameter = 'other_designation' + suffix
        otherDesignation = request.GET.get(otherDesignationParameter)
        if otherDesignation is not None:
            break

    if otherDesignation:
        queryFilter[otherDesignationParameter] = otherDesignation


    objectType = None
    objectType = request.GET.get('object_classification')
    try:
        objectType = int(objectType)
    except ValueError as e:
        objectType = None
    except TypeError as e:
        objectType = None

    if objectType:
        queryFilter[prefix + 'object_classification'] = objectType

    sherlockType = None
    sherlockType = request.GET.get('sherlockClassification')

    if sherlockType:
        queryFilter[prefix + 'sherlockClassification'] = sherlockType

    specType = None
    specType = request.GET.get('observation_status')

    if specType:
        queryFilter[prefix + 'observation_status'] = specType

    xt = None
    xtParameter = 'xt'

    for suffix in ['__isnull', '']:
        xtParameter = 'xt' + suffix
        xt = request.GET.get(xtParameter)
        if xt is not None:
            break

    try:
        xt = int(xt)
    except ValueError as e:
        xt = None
    except TypeError as e:
        xt = None

    if xt:
        queryFilter[prefix + xtParameter] = xt

    return queryFilter


# 2023-06-22 KWS Added helper to get GW event parameters
def filterGetGWParameters(request, queryFilter, prefix = ''):
    """filterGetParameters.

    Args:
        request:
        queryFilter:
        prefix:
    """


    gwEvent = None
    gwEventParameter = 'gravity_event_id'

    gwEvent = request.GET.get('gwevent')

    if gwEvent:
        queryFilter[prefix + gwEventParameter + '__contains'] = gwEvent


    enclosingContour = None
    enclosingContourParameter = 'enclosing_contour'
    for suffix in ['__lt', '__lte', '__gt', '__gte', '']:
        enclosingContourParameter = 'enclosing_contour' + suffix
        enclosingContour = request.GET.get(enclosingContourParameter)
        if enclosingContour is not None:
            break

    try:
        enclosingContour = float(enclosingContour)
    except ValueError as e:
        enclosingContour = None
    except TypeError as e:
        enclosingContour = None

    if enclosingContour:
        queryFilter[prefix + enclosingContourParameter] = enclosingContour

    daysSinceEvent = None
    daysSinceEventParameter = 'days_since_event'
    for suffix in ['__lt', '__lte', '__gt', '__gte', '']:
        daysSinceEventParameter = 'days_since_event' + suffix
        daysSinceEvent = request.GET.get(daysSinceEventParameter)
        if daysSinceEvent is not None:
            break

    try:
        daysSinceEvent = float(daysSinceEvent)
    except ValueError as e:
        daysSinceEvent = None
    except TypeError as e:
        daysSinceEvent = None

    if daysSinceEvent:
        queryFilter[prefix + daysSinceEventParameter] = daysSinceEvent

    return queryFilter


def getDjangoTables2ImageTemplate(imageType):
    IMAGE_TEMPLATE = """<img id="stampimages" src="{{ MEDIA_URL }}images/data/{{ dbname }}/{{ record.tcs_images_id.whole_mjd }}/{{ record.tcs_images_id.%s }}.jpeg" alt="triplet" title="{{ record.tcs_images_id.pss_filename }}" onerror="this.src='{{ STATIC_URL }}images/image_not_available.jpeg';" height="200" />""" % imageType
    return IMAGE_TEMPLATE




from rest_framework.permissions import BasePermission, SAFE_METHODS

class IsApprovedUser(BasePermission):
    def has_permission(self, request, view):
        # Allow all safe methods (GET, OPTIONS, HEAD)
        if request.method in SAFE_METHODS:
            return True
        
        # Only allow POST if the user is authenticated, active, and staff
        return (request.user 
                and request.user.is_authenticated 
                and request.user.is_active 
                and request.user.is_staff)


from django.contrib.auth.models import User
from rest_framework import authentication
from rest_framework import exceptions
from re import match

class QueryAuthentication(authentication.TokenAuthentication):
    def authenticate(self, request):
        token = request.query_params.get('token')

        if not token:
            return None

        if not match("^\w+$", token):
            msg = 'Invalid token format.'
            raise exceptions.AuthenticationFailed(msg)

        return self.authenticate_credentials(token)



import datetime
import re
import json
import requests
from django.db import IntegrityError
from django.db import connection
from datetime import datetime
from gkutils.commonutils import coneSearchHTM, FULL, QUICK, COUNT, CAT_ID_RA_DEC_COLS, base26, Struct
from rest_framework import serializers
import sys
from atlas.apiutils import candidateddcApi, getObjectList
from atlas.apiutils import getVRAScoresList
from atlas.apiutils import getVRATodoList
from atlas.apiutils import getCustomListObjects
from atlas.apiutils import getVRARankList
from atlas.apiutils import getExternalCrossmatchesList
from django.core.exceptions import ObjectDoesNotExist

# 2024-01-29 KWS Need the model to do inserts.
from atlas.models import TcsVraScores
from atlas.models import AtlasDiffObjects
from atlas.models import TcsVraTodo
from atlas.models import TcsObjectGroups
from atlas.models import TcsObjectGroupDefinitions
from atlas.models import TcsVraRank
from atlas.models import TcsCrossMatchesExternal

#CAT_ID_RA_DEC_COLS['objects'] = [['objectId', 'ramean', 'decmean'], 1018]

REQUEST_TYPE_CHOICES = (
    ('count', 'Count'),
    ('all', 'All'),
    ('nearest', 'Nearest'),
)


class ConeSerializer(serializers.Serializer):
    ra = serializers.FloatField(required=True)
    dec = serializers.FloatField(required=True)
    radius = serializers.FloatField(required=True)
    requestType = serializers.ChoiceField(choices=REQUEST_TYPE_CHOICES)

    def save(self):

        ra = self.validated_data['ra']
        dec = self.validated_data['dec']
        radius = self.validated_data['radius']
        requestType = self.validated_data['requestType']

        # Get the authenticated user, if it exists.
        userId = 'unknown'
        request = self.context.get("request")
        if request and hasattr(request, "user"):
            userId = request.user

        if radius > 1000:
            replyMessage = "Max radius is 1000 arcsec."
            info = {"error": replyMessage}
            return info

        replyMessage = 'No object found ra=%.5f dec=%.5f radius=%.2f' % (ra, dec, radius)
        info = {"error": replyMessage}

        # Is there an object within RADIUS arcsec of this object? - KWS - need to fix the gkhtm code!!
        # For ATLAS QUICK does not work because of the `dec` syntax error problem. Use FULL until I figure out what
        # needs to be fixed.
        message, results = coneSearchHTM(ra, dec, radius, 'atlas_diff_objects', queryType=FULL, conn=connection, django=True)

        obj = None
        separation = None

        objectList = []
        if len(results) > 0:
            if requestType == "nearest":
                obj = results[0][1]['id']
                separation = results[0][0]
                info = {"object": obj, "separation": separation}
            elif requestType == "all":
                for row in results:
                    objectList.append({"object": row[1]["id"], "separation": row[0]})
                info = objectList
            elif requestType == "count":
                info = {'count': len(results)}
            else:
                info = {"error": "Invalid request type"}

        return info


class ObjectsSerializer(serializers.Serializer):
    objects = serializers.CharField(required=True)
    mjd = serializers.FloatField(required=False, default=None)

    def save(self):
        objects = self.validated_data['objects']
        mjd = self.validated_data['mjd']

        olist = []
        for tok in objects.split(','):
            olist.append(tok.strip())

        if len(olist) > 100:
            return {"info": "Max number of objects for each requests is 100"}
#        olist = olist[:10] # restrict to 10

        # Get the authenticated user, if it exists.
        userId = 'unknown'
        request = self.context.get("request")
        if request and hasattr(request, "user"):
            userId = request.user


        result = []
        for candidate in olist:
            result.append(candidateddcApi(request, candidate, mjdThreshold=mjd))
        return result


class ObjectListSerializer(serializers.Serializer):
    objectlistid = serializers.IntegerField(required=True)
    getcustomlist = serializers.BooleanField(required=False, default = False)
    datethreshold = serializers.DateTimeField(required=False, default=None)

    def save(self):
        objectlistid = self.validated_data['objectlistid']
        getcustomlist = self.validated_data['getcustomlist']
        datethreshold = self.validated_data['datethreshold']

        request = self.context.get("request")

        dateThreshold = None
        if datethreshold is not None:
            dateThreshold = self.validated_data['datethreshold']

        objectList = getObjectList(request, objectlistid, getCustomList = getcustomlist, dateThreshold = dateThreshold)
        return objectList


# 2024-01-17 KWS Insert a VRA row for an object. For the time being do this one at a time.
# 2024-02-21 KWS Changed required to False for all three prob values.
class VRAScoresSerializer(serializers.Serializer):
    objectid = serializers.IntegerField(required=True)
    preal = serializers.FloatField(required=False, default=None)
    pgal = serializers.FloatField(required=False, default=None)
    pfast = serializers.FloatField(required=False, default=None)
    # 2024-08-14 KWS Added 3 new values to the form for Rank.
    rank = serializers.FloatField(required=True)
    rank_alt1 = serializers.FloatField(required=False, default=None)
    rank_alt2 = serializers.FloatField(required=False, default=None)
    debug = serializers.BooleanField(required=False, default=False)
    insertdate = serializers.DateTimeField(required=False, default=None)


    def save(self):

        from django.conf import settings
        objectid = self.validated_data['objectid']
        preal = self.validated_data['preal']
        pgal = self.validated_data['pgal']
        pfast = self.validated_data['pfast']
        rank = self.validated_data['rank']
        rank_alt1 = self.validated_data['rank_alt1']
        rank_alt2 = self.validated_data['rank_alt2']
        insertdate = self.validated_data['insertdate']
        debug = self.validated_data['debug']

        insertDate = None
        if insertdate is not None:
            insertDate = self.validated_data['insertdate']

        replyMessage = 'Row created.'

        userId = 'unknown'
        request = self.context.get("request")
        if request and hasattr(request, "user"):
            userId = str(request.user)

        if not insertDate:
            insertDate = datetime.now()

        data = {'transient_object_id_id': objectid,
                'preal': preal,
                'pgal': pgal,
                'pfast': pfast,
                'rank': rank,
                'rank_alt1': rank_alt1,
                'rank_alt2': rank_alt2,
                'timestamp': insertDate,
                'debug': debug,
                'apiusername': userId}

        # Does the objectId actually exit - not allowed to comment on objects that don't exist!
        # This should really return a 404 message.
        try:
            transient = AtlasDiffObjects.objects.get(pk=objectid)
        except ObjectDoesNotExist as e:
            replyMessage = 'Object does not exist.'
            info = { "objectid": objectid, "info": replyMessage }
            return info

        ## Does the VRA row exist?
        #vra = None
        #try:
        #    vra = TcsVraScores.objects.get(transient_object_id_id=objectid, debug=debug)
        
        #except ObjectDoesNotExist as e:
        #    # That's OK - we'll create a new object
        #    pass
        
        #if vra:
        #    instance = vra
        #else:
        instance = TcsVraScores(**data)
        #try:

        #    if vra is not None:
        #        instance.preal = preal
        #        instance.pfast = pfast
        #        instance.pgal = pgal
        #        instance.timestamp = insertDate
        #        i = instance.save()
        #    else:
        i = instance.save()

        #except IntegrityError as e:
        #    replyMessage = 'Duplicate row. Cannot add row.'

        info = { "objectid": objectid, "info": replyMessage }
        return info



class VRAScoresListSerializer(serializers.Serializer):
    objects = serializers.CharField(required=False, default=None)
    debug = serializers.BooleanField(required=False, default=False)
    datethreshold = serializers.DateTimeField(required=False, default='1970-01-01')
    idthreshold = serializers.IntegerField(required=False, default=0)

    def save(self):
        objects = self.validated_data['objects']
        datethreshold = self.validated_data['datethreshold']
        debug = self.validated_data['debug']
        idthreshold = self.validated_data['idthreshold']

        request = self.context.get("request")

        olist = []

        if objects is not None:
            for tok in objects.split(','):
                olist.append(tok.strip())

        vraScoresList = getVRAScoresList(request, objects = olist, debug = debug, dateThreshold = datethreshold, idThreshold = idthreshold)
        return vraScoresList


# 2024-02-21 KWS Changed required to False for all three prob values.
class VRATodoSerializer(serializers.Serializer):
    objectid = serializers.IntegerField(required=True)
    insertdate = serializers.DateTimeField(required=False, default=None)

    import sys

    def save(self):

        from django.conf import settings
        objectid = self.validated_data['objectid']
        insertdate = self.validated_data['insertdate']

        insertDate = None
        if insertdate is not None:
            insertDate = self.validated_data['insertdate']

        replyMessage = 'Row created.'

        if not insertDate:
            insertDate = datetime.now()

        data = {'transient_object_id_id': objectid,
                'timestamp': insertDate}

        # Does the objectId actually exit - not allowed to comment on objects that don't exist!
        # This should really return a 404 message.
        try:
            transient = AtlasDiffObjects.objects.get(pk=objectid)
        except ObjectDoesNotExist as e:
            replyMessage = 'Object does not exist.'
            info = { "objectid": objectid, "info": replyMessage }
            return info

        try:
            instance = TcsVraTodo(**data)
            i = instance.save(force_insert=True)
            # NOTE: Inserting an object by setting the primary key actually REPLACES the object. Do we want this behaviour??
            #       The integrity error below never happens because I've now set the model with primary_key=True.
            #       To fix this I've added force_insert = True above.
        except IntegrityError as e:
            replyMessage = 'Duplicate row. Cannot add row.'

        info = { "objectid": objectid, "info": replyMessage }
        return info


class VRATodoListSerializer(serializers.Serializer):
    objects = serializers.CharField(required=False, default=None)
    datethreshold = serializers.DateTimeField(required=False, default='1970-01-01')
    idthreshold = serializers.IntegerField(required=False, default=0)

    def save(self):
        objects = self.validated_data['objects']
        datethreshold = self.validated_data['datethreshold']
        idthreshold = self.validated_data['idthreshold']

        request = self.context.get("request")

        olist = []

        if objects is not None:
            for tok in objects.split(','):
                olist.append(tok.strip())

        vraTodoList = getVRATodoList(request, objects = olist, dateThreshold = datethreshold, idThreshold = idthreshold)
        return vraTodoList


# 2024-02-21 KWS Changed required to False for all three prob values.
class TcsObjectGroupsSerializer(serializers.Serializer):
    objectid = serializers.IntegerField(required=True)
    objectgroupid = serializers.IntegerField(required=True)

    import sys

    def save(self):

        from django.conf import settings
        objectid = self.validated_data['objectid']
        objectGroupId = self.validated_data['objectgroupid']

        replyMessage = 'Row created.'

        # This is what gets inserted into the database.
        data = {'transient_object_id_id': objectid,
                'object_group_id': objectGroupId}

        # Does the objectId actually exit - not allowed to comment on objects that don't exist!
        # This should really return a 404 message.
        try:
            transient = AtlasDiffObjects.objects.get(pk=objectid)
        except ObjectDoesNotExist as e:
            replyMessage = 'Object does not exist.'
            info = { "objectid": objectid, "info": replyMessage }
            return info

        try:
            group = TcsObjectGroupDefinitions.objects.get(pk=objectGroupId)
        except ObjectDoesNotExist as e:
            replyMessage = 'Object group ID does not exist.'
            info = { "objectgroupid": objectGroupId, "info": replyMessage }
            return info

        try:
            instance = TcsObjectGroups(**data)
            i = instance.save(force_insert=True)
            # NOTE: Inserting an object by setting the primary key actually REPLACES the object. Do we want this behaviour??
            #       The integrity error below never happens because I've now set the model with primary_key=True.
            #       To fix this I've added force_insert = True above.
        except IntegrityError as e:
            replyMessage = 'Duplicate row. Cannot add row.'

        info = { "objectgroupid": objectid, "info": replyMessage }
        return info

# 2024-02-21 KWS Changed required to False for all three prob values.
class TcsObjectGroupsDeleteSerializer(serializers.Serializer):
    objectid = serializers.IntegerField(required=True)
    objectgroupid = serializers.IntegerField(required=True)

    import sys

    def save(self):

        from django.conf import settings
        objectid = self.validated_data['objectid']
        objectGroupId = self.validated_data['objectgroupid']

        replyMessage = 'Row deleted.'

        # This is what gets inserted into the database.
        data = {'transient_object_id_id': objectid,
                'object_group_id': objectGroupId}

        # Does the objectId actually exit - not allowed to comment on objects that don't exist!
        # This should really return a 404 message.
        try:
            transient = AtlasDiffObjects.objects.get(pk=objectid)
        except ObjectDoesNotExist as e:
            replyMessage = 'Object does not exist.'
            info = { "objectid": objectid, "info": replyMessage }
            return info

        try:
            group = TcsObjectGroupDefinitions.objects.get(pk=objectGroupId)
        except ObjectDoesNotExist as e:
            replyMessage = 'Object group ID does not exist.'
            info = { "objectgroupid": objectGroupId, "info": replyMessage }
            return info

        try:
            instance = TcsObjectGroups.objects.get(transient_object_id__id = objectid, object_group_id = objectGroupId)
        except ObjectDoesNotExist as e:
            replyMessage = 'Object group ID does not exist or object ID does not exist.'
            info = { "objectgroupid": objectGroupId, "objectid": objectid, "info": replyMessage }
            return info

        i = instance.delete()
        #replyMessage = 'Duplicate row. Cannot add row.'

        info = { "objectgroupid": objectid, "info": replyMessage }
        return info

class TcsObjectGroupsListSerializer(serializers.Serializer):
    objectid = serializers.IntegerField(required=False, default=None)
    objectgroupid = serializers.IntegerField(required=False, default=None)

    def save(self):
        objectid = self.validated_data['objectid']
        objectGroupId = self.validated_data['objectgroupid']

        request = self.context.get("request")

        customListObjects = getCustomListObjects(request, objectid, objectGroupId)
        return customListObjects


# 2024-05-22 KWS Added VRARank and VRARankList Serializers.
class VRARankSerializer(serializers.Serializer):
    objectid = serializers.IntegerField(required=True)
    rank = serializers.FloatField(required=True)
    rank_alt1 = serializers.FloatField(required=False, default=None)
    rank_alt2= serializers.FloatField(required=False, default=None)
    insertdate = serializers.DateTimeField(required=False, default=None)

    import sys

    def save(self):

        from django.conf import settings
        objectid = self.validated_data['objectid']
        insertdate = self.validated_data['insertdate']
        rank = self.validated_data['rank']
        rank_alt1 = self.validated_data['rank_alt1']
        rank_alt2 = self.validated_data['rank_alt2']

        insertDate = None
        if insertdate is not None:
            insertDate = self.validated_data['insertdate']

        replyMessage = 'Row created.'

        if not insertDate:
            insertDate = datetime.now()

        data = {'transient_object_id_id': objectid,
                'rank': rank,
                'rank_alt1': rank_alt1,
                'rank_alt2': rank_alt2,
                'timestamp': insertDate}

        # Does the objectId actually exit - not allowed to comment on objects that don't exist!
        # This should really return a 404 message.
        try:
            transient = AtlasDiffObjects.objects.get(pk=objectid)
        except ObjectDoesNotExist as e:
            replyMessage = 'Object does not exist.'
            info = { "objectid": objectid, "info": replyMessage }
            return info

        try:
            instance = TcsVraRank(**data)
            # Do not use force_insert=True here. We really want to update any existing row
            # or insert if the row doesn't exist.
            i = instance.save()

        except IntegrityError as e:
            # Should never happen!
            sys.stderr.write(str(e))
            replyMessage = 'Duplicate row. Cannot add row.'

        info = { "objectid": objectid, "info": replyMessage }
        return info


class VRARankListSerializer(serializers.Serializer):
    objects = serializers.CharField(required=False, default=None)
    datethreshold = serializers.DateTimeField(required=False, default='1970-01-01')

    def save(self):
        objects = self.validated_data['objects']
        datethreshold = self.validated_data['datethreshold']

        request = self.context.get("request")

        olist = []

        if objects is not None:
            for tok in objects.split(','):
                olist.append(tok.strip())

        vraRankList = getVRARankList(request, objects = olist, dateThreshold = datethreshold)
        return vraRankList

# 2024-09-24 KWS Added tcs_cross_matches_external viewer. We want to extract any ATLAS objects associated
#                with any named external courses.

class ExternalCrossmatchesListSerializer(serializers.Serializer):
    # We could send ATLAS IDs or any external crossmatch IDs, like TNS names.
    externalObjects = serializers.CharField(required=False, default=None)

    def save(self):
        externalObjects = self.validated_data['externalObjects']

        request = self.context.get("request")

        olist = []

        if externalObjects is not None:
            for tok in externalObjects.split(','):
                olist.append(tok.strip())

        externalCrossmatchesList = getExternalCrossmatchesList(request, externalObjects = olist)
        return externalCrossmatchesList

# 2024-09-24 KWS Added new serializer to updated the detection_list_id of an object in order
#                to be able to "snooze" and "unsnooze" it. We'll use the possible list as
#                snooze list.
class ObjectDetectionListSerializer(serializers.Serializer):
    objectid = serializers.IntegerField(required=True)
    # Only allow the object list to be updated
    objectlist = serializers.IntegerField(required=True)
    # Updates the date_modified field for an object too.
    insertdate = serializers.DateTimeField(required=False, default=None)

    import sys

    def save(self):

        from django.conf import settings
        objectid = self.validated_data['objectid']
        objectlist = self.validated_data['objectlist']
        insertdate = self.validated_data['insertdate']

        insertDate = None
        if insertdate is not None:
            insertDate = self.validated_data['insertdate']

        replyMessage = 'Row created.'

        if not insertDate:
            insertDate = datetime.now()

        data = {'detection_list_id_id': objectlist,
                'date_modified': insertDate}

        # Does the objectId actually exit - not allowed to comment on objects that don't exist!
        # This should really return a 404 message.
        try:
            transient = AtlasDiffObjects.objects.get(pk=objectid)
        except ObjectDoesNotExist as e:
            replyMessage = 'Object does not exist.'
            info = { "objectid": objectid, "info": replyMessage }
            return info

        try:
            for key, value in data.items():
                sys.stderr.write("\nkey=%s\n" % key)
                sys.stderr.write("\nvalue=%s\n" % value)
                if key=='detection_list_id_id' and value not in (3,4):
                    replyMessage = 'Error updating row.'
                    info = { "objectid": objectid, "info": replyMessage }
                    return info
                else:
                    setattr(transient, key, value)
            transient.save()

        except IntegrityError as e:
            replyMessage = 'Error updating row.'

        info = { "objectid": objectid, "info": replyMessage }
        return info




from requests.exceptions import ConnectionError as RequestsConnectionError
from requests.exceptions import Timeout as RequestsConnectionTimeoutError
from django.conf import settings
from atlas.models import AtlasDiffObjects
from atlas.models import AtlasForcedPhotometry
from atlas.models import TcsGravityEventAnnotations
from atlas.models import SherlockClassifications
from atlas.models import SherlockCrossmatches
from atlas.models import TcsObjectComments
from atlas.models import TcsCrossMatchesExternal
from atlas.models import AtlasDetectionsddc
from atlas.models import TcsVraScores
from atlas.models import TcsVraTodo
from atlas.models import TcsObjectGroups
from atlas.models import TcsVraRank
from django.forms.models import model_to_dict
from .lightcurvequeries import *
from .views import followupClassList
from .views import WebViewUserDefined
from .views import AtlasDetectionsddcTable
from .commonqueries import getNonDetectionsUsingATLASFootprint, ATLAS_METADATADDC, filterWhereClauseddc, LC_POINTS_QUERY_ATLAS_DDC, FILTERS, getLightcurvePoints, getLightcurvePointsDDCAPI, getNonDetectionsUsingATLASFootprintAPI
from .dbviews import CustomLCBlanks, CustomLCPoints, CustomLCPoints2, CustomLCBlanks2

from .views import LC_LIMITS, LC_LIMITS_MD
import numpy as np
import sys
from django.core.exceptions import ObjectDoesNotExist

def candidateddcApi(request, atlas_diff_objects_id, mjdThreshold = None):
    """candidateddcApi.

    Args:
        request:
        atlas_diff_objects_id:
        mjdThreshold:
    """

    from django.db import connection
    import sys

    # 2021-10-21 KWS Use the Lasair API to do a cone search so we can check for nearby ZTF objects
    from lasair import LasairError, lasair_client as lasair

    transient = AtlasDiffObjects.objects.get(pk=atlas_diff_objects_id)

    # 2017-03-21 KWS Get Gravity Wave annotations and Sherlock Classifications
    sc = SherlockClassifications.objects.filter(transient_object_id_id = transient.id)
    # 2017-11-07 KWS Get Sherlock Crossmatches
    sx = SherlockCrossmatches.objects.filter(transient_object_id_id = transient.id)
    gw = TcsGravityEventAnnotations.objects.filter(transient_object_id_id = transient.id).filter(enclosing_contour__lt=100)
    #existingComments = TcsObjectComments.objects.filter(transient_object_id = transient.id).order_by('date_inserted')

    # 2013-10-30 KWS Get external crossmatches if they exist
    externalXMs = TcsCrossMatchesExternal.objects.filter(transient_object_id = transient.id).exclude(matched_list = 'Transient Name Server').order_by('external_designation')

    # 2019-10-18 KWS Get TNS crossmatch if it exists. Yes - another unnecessary hit to the database, but quick.
    tnsXMs = TcsCrossMatchesExternal.objects.filter(transient_object_id = transient.id, matched_list = 'Transient Name Server')

    dbName = settings.DATABASES['default']['NAME'].replace('_django', '')

    try:
        detectionLimits = LC_LIMITS[dbName]
    except KeyError as e:
        # Default detections limits for medium deep
        detectionLimits = LC_LIMITS_MD

    if mjdThreshold is not None:
        forcedPhotometry = AtlasForcedPhotometry.objects.filter(atlas_object_id = transient.id).filter(mjd_obs__gte=mjdThreshold).order_by('mjd_obs')
    else:
        forcedPhotometry = AtlasForcedPhotometry.objects.filter(atlas_object_id = transient.id).order_by('mjd_obs')


    recsExperiment = getLightcurvePointsDDCAPI(transient.id, djangoRawObject = CustomLCPoints2, lcQuery=LC_POINTS_QUERY_ATLAS_DDC + filterWhereClauseddc(FILTERS), conn = connection, mjdThreshold = mjdThreshold)
    blanksExperiment = getNonDetectionsUsingATLASFootprintAPI(recsExperiment, djangoRawObject = CustomLCBlanks2, ndQuery=ATLAS_METADATADDC, filterWhereClause = filterWhereClauseddc, catalogueName = 'atlas_metadataddc', conn = connection, mjdThreshold = mjdThreshold, transient = transient)


    # 2024-02-05 KWS The model_to_dict method doesn't pull out any @property methods
    #                from the model, so need to iterate through and add them. Since
    #                we need to iterate anyway, this should not slow things down.
    forcedWithProperties = []
    for f in forcedPhotometry:
        fp = model_to_dict(f)
        fp['uJy'] = f.uJy
        fp['duJy'] = f.duJy
        fp['texp'] = f.texp
        forcedWithProperties.append(fp)

    data = {
            'object': model_to_dict(transient),
            'lc': [model_to_dict(r) for r in recsExperiment],
            'lcnondets': [model_to_dict(b) for b in blanksExperiment],
            'fp': forcedWithProperties,
            'sherlock_crossmatches': [model_to_dict(s) for s in sx],
            'sherlock_classifications': [model_to_dict(s) for s in sc],
            'tns_crossmatches': [model_to_dict(t) for t in tnsXMs],
            'external_crossmatches': [model_to_dict(e) for e in externalXMs]
           }

    return data


def getObjectList(request, listId, getCustomList = False, dateThreshold = None):

    querySet = None

    if getCustomList:
        if listId > 0 and listId <= 100:
            if dateThreshold is not None:
                querySet = WebViewUserDefined.objects.filter(object_group_id = listId, followup_flag_date__gt = dateThreshold)
            else:
                querySet = WebViewUserDefined.objects.filter(object_group_id = listId)
    else:
        # There are currently 11 valid lists.
        if listId >= 0 and listId <= 11:
            if dateThreshold is not None:
                querySet = followupClassList[int(listId)].objects.filter(followup_flag_date__gt = dateThreshold)
            else:
                querySet = followupClassList[int(listId)].objects.all()

    objectList = []

    if querySet is not None:
        for row in querySet:
            objectList.append(model_to_dict(row))

    return objectList


def getVRAScoresList(request, objects = [], debug = False, dateThreshold = None, idThreshold = 0):

    vraScoresList = []

    # If we specify objects then ignore any thresholds. Deprecated flag is common to all objects.
    # If you don't want a common debug flag, then request objects one at a time!
    if len(objects) > 0:
        for objectid in objects:
            try:
                oid = int(objectid)
            except ValueError as e:
                continue

            try:
                querySet = TcsVraScores.objects.filter(transient_object_id_id=oid, debug=debug)
                if querySet is not None:
                    for vra in querySet:
                        vraScoresList.append(model_to_dict(vra))
            except ObjectDoesNotExist as e:
                # Silent fail. No objects returned if the object does not exist.
                pass
    else:
        querySet = TcsVraScores.objects.filter(pk__gte=idThreshold).filter(timestamp__gte=dateThreshold)
        #querySet = TcsVraScores.objects.all()
        if querySet is not None:
            for vra in querySet:
                vraScoresList.append(model_to_dict(vra))

    return vraScoresList


# 2024-05-07 KWS Added getVRATodoList
def getVRATodoList(request, objects = [], dateThreshold = None, idThreshold = 0):

    vraTodoList = []

    # If we specify objects then ignore any thresholds. Deprecated flag is common to all objects.
    if len(objects) > 0:
        for objectid in objects:
            try:
                oid = int(objectid)
            except ValueError as e:
                continue

            try:
                querySet = TcsVraTodo.objects.filter(transient_object_id_id=oid)
                if querySet is not None:
                    for vra in querySet:
                        vraTodoList.append(model_to_dict(vra))
            except ObjectDoesNotExist as e:
                # Silent fail. No objects returned if the object does not exist.
                pass
    else:
        querySet = TcsVraTodo.objects.filter(pk__gte=idThreshold).filter(timestamp__gte=dateThreshold)
        #querySet = TcsVraTodo.objects.all()
        if querySet is not None:
            for vra in querySet:
                vraTodoList.append(model_to_dict(vra))

    return vraTodoList


# 2024-05-20 KWS Added getCustomListObjects
def getCustomListObjects(request, objectid = None, objectgroupid = None):

    customListObjects = []

    if objectid is None and objectgroupid is not None:
        querySet = TcsObjectGroups.objects.filter(object_group_id = objectgroupid)
    elif objectid is not None and objectgroupid is None:
        querySet = TcsObjectGroups.objects.filter(transient_object_id__id = objectid)
    else:
        querySet = TcsObjectGroups.objects.all()

    if querySet is not None:
        for row in querySet:
            customListObjects.append(model_to_dict(row))

    return customListObjects

# 2024-05-07 KWS Added getVRARankList
def getVRARankList(request, objects = [], dateThreshold = None, idThreshold = 0):

    vraRankList = []

    # If we specify objects then ignore any thresholds. Deprecated flag is common to all objects.
    if len(objects) > 0:
        for objectid in objects:
            try:
                oid = int(objectid)
            except ValueError as e:
                continue

            try:
                querySet = TcsVraRank.objects.filter(transient_object_id_id=oid)
                if querySet is not None:
                    for vra in querySet:
                        vraRankList.append(model_to_dict(vra))
            except ObjectDoesNotExist as e:
                # Silent fail. No objects returned if the object does not exist.
                pass
    else:
        querySet = TcsVraRank.objects.filter(pk__gte=idThreshold).filter(timestamp__gte=dateThreshold)
        #querySet = TcsVraRank.objects.all()
        if querySet is not None:
            for vra in querySet:
                vraRankList.append(model_to_dict(vra))

    return vraRankList


# 2024-09-24 KWS Added getExternalCrossmatches
def getExternalCrossmatchesList(request, externalObjects = []):

    externalCrossmatchesList = []

    for xm in externalObjects:
        miniList = []
        querySet = TcsCrossMatchesExternal.objects.filter(external_designation=xm)
        if querySet is not None:
            for x in querySet:
                miniList.append(model_to_dict(x))
            externalCrossmatchesList.append(miniList)

    return externalCrossmatchesList



# This is an auto-generated Django model module.
# You'll have to do the following manually to clean this up:
#   * Rearrange models' order
#   * Make sure each model has one field with primary_key=True
#   * Remove `managed = False` lines if you wish to allow Django to create, modify, and delete the table
# Feel free to rename the models, but don't rename db_table values or field names.
#
# Also note: You'll have to insert the output of 'django-admin.py sqlcustom [app_label]'
# into your database.
import sys

from django.db import models

from gkutils.commonutils import FLAGS, PROCESSING_FLAGS, getFlagDefs, ra_to_sex, dec_to_sex, getDateFractionMJD, getMJDFromSqlDate


class TcsDetectionLists(models.Model):
    """TcsDetectionLists.
    """

    id = models.IntegerField(primary_key=True)  # AutoField?
    name = models.CharField(max_length=20)
    description = models.CharField(max_length=80, blank=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_detection_lists'


class TcsImageGroups(models.Model):
    """TcsImageGroups.
    """

    id = models.BigIntegerField(primary_key=True)
    name = models.CharField(unique=True, max_length=255)
    group_type = models.IntegerField(blank=True, null=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_image_groups'


class AtlasMetadata(models.Model):
    """AtlasMetadata.
    """

    id = models.BigIntegerField(primary_key=True)
    filename = models.CharField(max_length=255, blank=True)
    expname = models.CharField(max_length=255, blank=True)
    object = models.CharField(max_length=255, blank=True)
    mjd_obs = models.FloatField(blank=True, null=True)
    filter = models.CharField(max_length=10, blank=True)
    pa = models.FloatField(blank=True, null=True)
    exptime = models.FloatField(blank=True, null=True)
    nx = models.IntegerField(blank=True, null=True)
    ny = models.IntegerField(blank=True, null=True)
    rad = models.IntegerField(blank=True, null=True)
    scale = models.FloatField(blank=True, null=True)
    longitude = models.FloatField(blank=True, null=True)
    latitude = models.FloatField(blank=True, null=True)
    elev = models.FloatField(blank=True, null=True)
    ra = models.FloatField(blank=True, null=True)
    dec = models.FloatField(blank=True, null=True)
    rarms = models.FloatField(blank=True, null=True)
    decrms = models.FloatField(blank=True, null=True)
    zp = models.FloatField(blank=True, null=True)
    skymag = models.FloatField(blank=True, null=True)
    cloud = models.FloatField(blank=True, null=True)
    mag5sig = models.FloatField(blank=True, null=True)
    az = models.FloatField(blank=True, null=True)
    alt = models.FloatField(blank=True, null=True)
    lambda_field = models.FloatField(db_column='lambda', blank=True, null=True)  # Field renamed because it was a Python reserved word.
    beta = models.FloatField(blank=True, null=True)
    sunelong = models.FloatField(blank=True, null=True)
    htm16id = models.BigIntegerField(db_column='htm16ID', blank=True, null=True)  # Field name made lowercase.
    input = models.CharField(max_length=255, blank=True)
    reference = models.CharField(max_length=255, blank=True)
    date_inserted = models.DateTimeField()

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'atlas_metadata'


# 2018-09-20 KWS Added 5 new columns, apfit, bckgnd, gain, psfphi, ddcver.
class AtlasMetadataddc(models.Model):
    """AtlasMetadataddc.
    """

    id = models.BigIntegerField(primary_key=True)
    filename = models.CharField(max_length=255, blank=True, null=True)
    obs = models.CharField(unique=True, max_length=60, blank=True, null=True)
    obj = models.CharField(max_length=60, blank=True, null=True)
    filt = models.CharField(max_length=10, blank=True, null=True)
    mjd = models.FloatField(blank=True, null=True)
    texp = models.FloatField(blank=True, null=True)
    ra = models.FloatField(blank=True, null=True)
    dec = models.FloatField(blank=True, null=True)
    pa = models.FloatField(blank=True, null=True)
    nx = models.SmallIntegerField(blank=True, null=True)
    ny = models.SmallIntegerField(blank=True, null=True)
    rad = models.IntegerField(blank=True, null=True)
    fwmaj = models.FloatField(blank=True, null=True)
    fwmin = models.FloatField(blank=True, null=True)
    psfpa = models.FloatField(blank=True, null=True)
    scale = models.FloatField(blank=True, null=True)
    long = models.FloatField(blank=True, null=True)
    lat = models.FloatField(blank=True, null=True)
    elev = models.FloatField(blank=True, null=True)
    rarms = models.FloatField(blank=True, null=True)
    decrms = models.FloatField(blank=True, null=True)
    magzpt = models.FloatField(blank=True, null=True)
    skymag = models.FloatField(blank=True, null=True)
    cloud = models.FloatField(blank=True, null=True)
    mag5sig = models.FloatField(blank=True, null=True)
    az = models.FloatField(blank=True, null=True)
    alt = models.FloatField(blank=True, null=True)
    lambda_field = models.FloatField(db_column='lambda', blank=True, null=True)  # Field renamed because it was a Python reserved word.
    beta = models.FloatField(blank=True, null=True)
    sunelong = models.FloatField(blank=True, null=True)
    apfit = models.FloatField(blank=True, null=True)
    bckgnd = models.IntegerField(blank=True, null=True)
    gain = models.FloatField(blank=True, null=True)
    psfphi = models.FloatField(blank=True, null=True)
    ddcver = models.CharField(max_length=30, blank=True, null=True)
    input = models.CharField(max_length=255, blank=True, null=True)
    reference = models.CharField(max_length=255, blank=True, null=True)
    htm16id = models.BigIntegerField(db_column='htm16ID', blank=True, null=True)  # Field name made lowercase.
    date_inserted = models.DateTimeField()

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'atlas_metadataddc'
        unique_together = (('filename', 'mjd'),)


class TcsImages(models.Model):
    """TcsImages.
    """

    id = models.BigIntegerField(primary_key=True)
    target = models.CharField(max_length=255)
    ref = models.CharField(max_length=255, blank=True)
    diff = models.CharField(max_length=255, blank=True)
    mjd_obs = models.FloatField()

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_images'

    @property
    def whole_mjd(self):
        """whole_mjd.
        """
        # 2022-01-01 KWS Use the exposure name buried in the 3rd field and take the MJD which is
        #                always characters 3 to 7. DO NOT use MJD - since in S. Africa, the int MJD
        #                at the start of the night is always the previous day.
        m = None
        if self.diff is not None:
            m = int(self.diff.split('_')[2][3:8])
        return m


class AtlasDiffObjects(models.Model):
    """AtlasDiffObjects.
    """

    id = models.BigIntegerField(primary_key=True)
    followup_id = models.IntegerField(blank=True, null=True)
    followup_flag_date = models.DateField(blank=True, null=True)
    #detection_id = models.IntegerField()
    # Note the quoted model name in the foreign key. This model has not yet been defined.
    detection_id = models.ForeignKey('AtlasDiffDetections', to_field='id', db_column='detection_id', on_delete=models.CASCADE)
    ra = models.FloatField()
    dec = models.FloatField()
    object_classification = models.IntegerField(blank=True, null=True)
    sherlockClassification = models.CharField(max_length=120, blank=True)
    observation_status = models.CharField(max_length=40, blank=True)
    htm16id = models.BigIntegerField(db_column='htm16ID')  # Field name made lowercase.
    jtindex = models.IntegerField(blank=True, null=True)
    images_id = models.ForeignKey(TcsImages, to_field='id', db_column='images_id', on_delete=models.CASCADE)
    date_inserted = models.DateTimeField()
    date_modified = models.DateTimeField(blank=True, null=True)
    processing_flags = models.IntegerField(blank=True, null=True)
    updated_by = models.CharField(max_length=40, blank=True)
    followup_priority = models.IntegerField(blank=True, null=True)
    external_reference_id = models.CharField(max_length=40, blank=True)
    detection_list_id = models.ForeignKey(TcsDetectionLists, null=True, to_field='id', db_column='detection_list_id', on_delete=models.CASCADE)
    survey_field = models.CharField(max_length=10, blank=True)
    followup_counter = models.IntegerField(blank=True, null=True)
    atlas_designation = models.CharField(max_length=40, blank=True)
    other_designation = models.CharField(max_length=40, blank=True)
    current_trend = models.CharField(max_length=40, blank=True)
    local_comments = models.CharField(max_length=765, blank=True)
    ndetections = models.IntegerField(blank=True, null=True)
    vra = models.FloatField(blank=True, null=True, db_column='realbogus_factor')
    rb_pix = models.FloatField(blank=True, null=True, db_column='zooniverse_score')

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'atlas_diff_objects'

    @property
    def decode_flag_bits(self):
        """decode_flag_bits.
        """
        object_definition = getFlagDefs(self.object_classification, FLAGS, delimiter = ' ')
        return object_definition

    @property
    def decode_processing_flags_bits(self):
        """decode_processing_flags_bits.
        """
        pf = getFlagDefs(self.processing_flags, PROCESSING_FLAGS, delimiter = ' ')
        return pf

    @property
    def ra_sex(self):
        """ra_sex.
        """
        ra_in_sex = ra_to_sex (self.ra)
        return ra_in_sex

    @property
    def dec_sex(self):
        """dec_sex.
        """
        dec_in_sex = dec_to_sex (self.dec)
        return dec_in_sex


class AtlasDiffDetections(models.Model):
    """AtlasDiffDetections.
    """

    id = models.BigIntegerField(primary_key=True)
    atlas_metadata_id = models.ForeignKey(AtlasMetadata, to_field='id', db_column='atlas_metadata_id', on_delete=models.CASCADE)
    atlas_object_id = models.ForeignKey(AtlasDiffObjects, to_field='id', db_column='atlas_object_id', on_delete=models.CASCADE)
    tphot_id = models.IntegerField()
    x = models.FloatField(blank=True, null=True)
    y = models.FloatField(blank=True, null=True)
    peakval = models.FloatField(blank=True, null=True)
    skyval = models.FloatField(blank=True, null=True)
    peakfit = models.FloatField(blank=True, null=True)
    dpeak = models.FloatField(blank=True, null=True)
    skyfit = models.FloatField(blank=True, null=True)
    flux = models.FloatField(blank=True, null=True)
    dflux = models.FloatField(blank=True, null=True)
    major = models.FloatField(blank=True, null=True)
    minor = models.FloatField(blank=True, null=True)
    phi = models.FloatField(blank=True, null=True)
    err = models.FloatField(blank=True, null=True)
    chi_n = models.FloatField(db_column='chi_N', blank=True, null=True)  # Field name made lowercase.
    ra = models.FloatField()
    dec = models.FloatField()
    mag = models.FloatField(blank=True, null=True)
    dm = models.FloatField(blank=True, null=True)
    peak = models.FloatField(blank=True, null=True)
    sky = models.FloatField(blank=True, null=True)
    varkrn = models.FloatField(blank=True, null=True)
    pstar = models.FloatField(blank=True, null=True)
    pkast = models.FloatField(blank=True, null=True)
    preal = models.FloatField(blank=True, null=True)
    star = models.IntegerField(blank=True, null=True)
    dstar = models.IntegerField(blank=True, null=True)
    mstar = models.FloatField(blank=True, null=True)
    kast = models.IntegerField(blank=True, null=True)
    dkast = models.IntegerField(blank=True, null=True)
    htm16id = models.BigIntegerField(db_column='htm16ID')  # Field name made lowercase.
    jtindex = models.IntegerField(blank=True, null=True)
    date_inserted = models.DateTimeField()
    date_modified = models.DateTimeField(blank=True, null=True)
    image_group_id = models.ForeignKey(TcsImageGroups, to_field='id', db_column='image_group_id', on_delete=models.CASCADE)
    quality_threshold_pass = models.IntegerField(blank=True, null=True)
    deprecated = models.IntegerField(blank=True, null=True)
    realbogus_factor = models.FloatField(blank=True, null=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'atlas_diff_detections'


class AtlasDetectionsddc(models.Model):
    """AtlasDetectionsddc.
    """

    id = models.BigIntegerField(primary_key=True)
    atlas_metadata_id = models.ForeignKey(AtlasMetadataddc, to_field='id', db_column='atlas_metadata_id', on_delete=models.CASCADE)
    atlas_object_id = models.ForeignKey(AtlasDiffObjects, to_field='id', db_column='atlas_object_id', on_delete=models.CASCADE)
    det_id = models.IntegerField()
    ra = models.FloatField(blank=True, null=True)
    dec = models.FloatField(blank=True, null=True)
    mag = models.FloatField(blank=True, null=True)
    dmag = models.FloatField(blank=True, null=True)
    x = models.FloatField(blank=True, null=True)
    y = models.FloatField(blank=True, null=True)
    major = models.FloatField(blank=True, null=True)
    minor = models.FloatField(blank=True, null=True)
    phi = models.FloatField(blank=True, null=True)
    det = models.IntegerField(blank=True, null=True)
    chin = models.FloatField(blank=True, null=True)
    pvr = models.SmallIntegerField(blank=True, null=True)
    ptr = models.SmallIntegerField(blank=True, null=True)
    pmv = models.SmallIntegerField(blank=True, null=True)
    pkn = models.SmallIntegerField(blank=True, null=True)
    pno = models.SmallIntegerField(blank=True, null=True)
    pbn = models.SmallIntegerField(blank=True, null=True)
    pcr = models.SmallIntegerField(blank=True, null=True)
    pxt = models.SmallIntegerField(blank=True, null=True)
    psc = models.SmallIntegerField(blank=True, null=True)
    dup = models.IntegerField(blank=True, null=True)
    wpflx = models.FloatField(blank=True, null=True)
    dflx = models.FloatField(blank=True, null=True)
    date_modified = models.DateTimeField(blank=True, null=True)
    image_group_id = models.BigIntegerField(blank=True, null=True)
    quality_threshold_pass = models.IntegerField(blank=True, null=True)
    deprecated = models.IntegerField(blank=True, null=True)
    realbogus_factor = models.FloatField(blank=True, null=True)
    htm16id = models.BigIntegerField(db_column='htm16ID', blank=True, null=True)  # Field name made lowercase.
    date_inserted = models.DateTimeField()

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'atlas_detectionsddc'



# 2018-01-30 KWS Added major and minor
# 2018-09-21 KWS Added apfit
# 2018-09-25 KWS Added foreign key to AtlasMetadataddc so we can pull out
#                the correct exposure time and apfit values.
class AtlasForcedPhotometry(models.Model):
    """AtlasForcedPhotometry.
    """

    id = models.BigIntegerField(primary_key=True)
    atlas_object_id = models.ForeignKey(AtlasDiffObjects, to_field='id', db_column='atlas_object_id', on_delete=models.CASCADE)
    expname = models.ForeignKey(AtlasMetadataddc, to_field='obs', db_column='expname', on_delete=models.CASCADE)
    #expname = models.CharField(max_length=255, blank=True)
    mjd_obs = models.FloatField(blank=True, null=True)
    ra = models.FloatField(blank=True, null=True)
    dec = models.FloatField(blank=True, null=True)
    filter = models.CharField(max_length=10, blank=True)
    mag = models.FloatField(blank=True, null=True)
    dm = models.FloatField(blank=True, null=True)
    snr = models.FloatField(blank=True, null=True)
    zp = models.FloatField(blank=True, null=True)
    x = models.FloatField(blank=True, null=True)
    y = models.FloatField(blank=True, null=True)
    peakval = models.FloatField(blank=True, null=True)
    skyval = models.FloatField(blank=True, null=True)
    peakfit = models.FloatField(blank=True, null=True)
    dpeak = models.FloatField(blank=True, null=True)
    skyfit = models.FloatField(blank=True, null=True)
    flux = models.FloatField(blank=True, null=True)
    dflux = models.FloatField(blank=True, null=True)
    dflux = models.FloatField(blank=True, null=True)
    chin = models.FloatField(blank=True, null=True)
    major = models.FloatField(blank=True, null=True)
    minor = models.FloatField(blank=True, null=True)
    snrdet = models.FloatField(blank=True, null=True)
    snrlimit = models.FloatField(blank=True, null=True)
    apfit = models.FloatField(blank=True, null=True)
    date_inserted = models.DateTimeField()
    limiting_mag = models.BooleanField(null=True, blank=True)
    redo_photometry = models.BooleanField(null=True, blank=True)


    # 2024-02-05 KWS Added uJy and duJy to the model definition, so the flux
    #                in microjanskys gets returned via the API.
    @property
    def uJy(self):
        exptime = self.expname.texp
        apfit = self.apfit
        if apfit is None:
            apfit = self.expname.apfit
        if self.zp is not None and self.major is not None and self.minor is not None and self.peakfit is not None and self.dpeak is not None and apfit is not None and exptime is not None and self.zp is not None:
            factor = 10**(-0.4*(self.zp+apfit-23.9))
            uJy = self.peakfit*self.major*self.minor/exptime*factor
        else:
            uJy = None
        return uJy


    @property
    def duJy(self):
        exptime = self.expname.texp
        apfit = self.apfit
        if apfit is None:
            apfit = self.expname.apfit
        if self.zp is not None and self.major is not None and self.minor is not None and self.peakfit is not None and self.dpeak is not None and apfit is not None and exptime is not None and self.zp is not None:
            factor = 10**(-0.4*(self.zp+apfit-23.9))
            duJy = self.dpeak*self.major*self.minor/exptime*factor
        else:
            duJy = None
        return duJy

    @property
    def texp(self):
        return self.expname.texp


    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'atlas_forced_photometry'


class TcsCatalogueTables(models.Model):
    """TcsCatalogueTables.
    """

    id = models.IntegerField(primary_key=True)  # AutoField?
    table_name = models.CharField(max_length=40)
    description = models.CharField(max_length=60)
    url = models.CharField(max_length=255, blank=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_catalogue_tables'


class TcsClassificationFlags(models.Model):
    """TcsClassificationFlags.
    """

    flag_id = models.IntegerField(primary_key=True)
    flag_name = models.CharField(max_length=30)
    description = models.CharField(max_length=80, blank=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_classification_flags'


class TcsClassificationHistory(models.Model):
    """TcsClassificationHistory.
    """

    id = models.BigIntegerField(primary_key=True)
    transient_object_id = models.ForeignKey(AtlasDiffObjects, to_field='id', db_column='transient_object_id', on_delete=models.CASCADE)
    last_classification = models.IntegerField()
    reclassification_date = models.DateTimeField()
    comments = models.CharField(max_length=255, blank=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_classification_history'


class TcsCrossMatches(models.Model):
    """TcsCrossMatches.
    """

    transient_object_id = models.ForeignKey(AtlasDiffObjects, to_field='id', db_column='transient_object_id', on_delete=models.CASCADE)
    catalogue_object_id = models.CharField(max_length=30)
    catalogue_table_id = models.IntegerField()
    search_parameters_id = models.IntegerField()
    separation = models.FloatField(blank=True, null=True)
    id = models.BigIntegerField(primary_key=True)
    z = models.FloatField(blank=True, null=True)
    scale = models.FloatField(blank=True, null=True)
    distance = models.FloatField(blank=True, null=True)
    distance_modulus = models.FloatField(blank=True, null=True)
    association_type = models.IntegerField(blank=True, null=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_cross_matches'


class TcsCrossMatchesExternal(models.Model):
    """TcsCrossMatchesExternal.
    """

    id = models.BigIntegerField(primary_key=True)
    transient_object_id = models.ForeignKey(AtlasDiffObjects, to_field='id', db_column='transient_object_id', on_delete=models.CASCADE)
    external_designation = models.CharField(max_length=60)
    type = models.CharField(max_length=40, blank=True)
    host_galaxy = models.CharField(max_length=60, blank=True)
    mag = models.FloatField(blank=True, null=True)
    discoverer = models.CharField(max_length=300, blank=True)
    matched_list = models.CharField(max_length=100)
    other_info = models.CharField(max_length=300, blank=True)
    separation = models.FloatField(blank=True, null=True)
    comments = models.CharField(max_length=300, blank=True)
    url = models.CharField(max_length=300, blank=True)
    host_z = models.FloatField(blank=True, null=True)
    object_z = models.FloatField(blank=True, null=True)
    disc_date = models.DateTimeField(blank=True, null=True)
    disc_filter = models.CharField(max_length=150, blank=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_cross_matches_external'


class TcsFollowupPhotometry(models.Model):
    """TcsFollowupPhotometry.
    """

    id = models.BigIntegerField(primary_key=True)
    transient_object_id = models.ForeignKey(AtlasDiffObjects, to_field='id', db_column='transient_object_id', on_delete=models.CASCADE)
    telescope_instrument_id = models.IntegerField()
    mjd = models.FloatField()
    filter = models.CharField(max_length=20, blank=True)
    mag = models.FloatField(blank=True, null=True)
    magerr = models.FloatField(blank=True, null=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_followup_photometry'


class TcsFollowupTelescopeInstruments(models.Model):
    """TcsFollowupTelescopeInstruments.
    """

    id = models.IntegerField(primary_key=True)  # AutoField?
    telescope_id = models.IntegerField()
    name = models.CharField(max_length=30)
    description = models.CharField(max_length=60)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_followup_telescope_instruments'


class TcsFollowupTelescopes(models.Model):
    """TcsFollowupTelescopes.
    """

    id = models.IntegerField(primary_key=True)  # AutoField?
    name = models.CharField(max_length=30)
    description = models.CharField(max_length=60)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_followup_telescopes'


class TcsForcedPhotometry(models.Model):
    """TcsForcedPhotometry.
    """

    id = models.BigIntegerField(primary_key=True)
    postage_stamp_request_id = models.BigIntegerField(blank=True, null=True)
    transient_object_id = models.ForeignKey(AtlasDiffObjects, to_field='id', db_column='transient_object_id', on_delete=models.CASCADE)
    rownum = models.IntegerField(blank=True, null=True)
    skycell = models.CharField(max_length=10, blank=True)
    mjd_obs = models.FloatField(blank=True, null=True)
    exptime = models.FloatField(blank=True, null=True)
    filter = models.CharField(max_length=20, blank=True)
    fpa_id = models.CharField(max_length=20, blank=True)
    x_psf = models.FloatField(blank=True, null=True)
    y_psf = models.FloatField(blank=True, null=True)
    x_psf_sig = models.FloatField(blank=True, null=True)
    y_psf_sig = models.FloatField(blank=True, null=True)
    posangle = models.FloatField(blank=True, null=True)
    pltscale = models.FloatField(blank=True, null=True)
    psf_inst_mag = models.FloatField(blank=True, null=True)
    psf_inst_mag_sig = models.FloatField(blank=True, null=True)
    psf_inst_flux = models.FloatField(blank=True, null=True)
    psf_inst_flux_sig = models.FloatField(blank=True, null=True)
    ap_mag = models.FloatField(blank=True, null=True)
    ap_mag_radius = models.FloatField(blank=True, null=True)
    peak_flux_as_mag = models.FloatField(blank=True, null=True)
    cal_psf_mag = models.FloatField(blank=True, null=True)
    cal_psf_mag_sig = models.FloatField(blank=True, null=True)
    ra_psf = models.FloatField()
    dec_psf = models.FloatField()
    sky = models.FloatField(blank=True, null=True)
    sky_sigma = models.FloatField(blank=True, null=True)
    psf_chisq = models.FloatField(blank=True, null=True)
    cr_nsigma = models.FloatField(blank=True, null=True)
    ext_nsigma = models.FloatField(blank=True, null=True)
    psf_major = models.FloatField(blank=True, null=True)
    psf_minor = models.FloatField(blank=True, null=True)
    psf_theta = models.FloatField(blank=True, null=True)
    psf_qf = models.FloatField(blank=True, null=True)
    psf_ndof = models.IntegerField(blank=True, null=True)
    psf_npix = models.IntegerField(blank=True, null=True)
    moments_xx = models.FloatField(blank=True, null=True)
    moments_xy = models.FloatField(blank=True, null=True)
    moments_yy = models.FloatField(blank=True, null=True)
    diff_npos = models.IntegerField(blank=True, null=True)
    diff_fratio = models.FloatField(blank=True, null=True)
    diff_nratio_bad = models.FloatField(blank=True, null=True)
    diff_nratio_mask = models.FloatField(blank=True, null=True)
    diff_nratio_all = models.FloatField(blank=True, null=True)
    flags = models.IntegerField(blank=True, null=True)
    n_frames = models.IntegerField(blank=True, null=True)
    padding = models.IntegerField(blank=True, null=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_forced_photometry'


# 2015-11-23 KWS Experiment with OneToOneField for id, since this is both
#                a primary key and a foreign key.
class TcsLatestObjectStats(models.Model):
    """TcsLatestObjectStats.
    """

    #id = models.OneToOneField(AtlasDiffObjects, on_delete=models.CASCADE, db_column='id', related_name='stats', primary_key=True, blank=True, null=True)
    id = models.OneToOneField(AtlasDiffObjects, on_delete=models.CASCADE, db_column='id', related_name='stats', primary_key=True)
    latest_mjd = models.FloatField(blank=True, null=True)
    latest_mag = models.FloatField(blank=True, null=True)
    latest_filter = models.CharField(max_length=80, blank=True)
    earliest_mjd = models.FloatField(blank=True, null=True)
    earliest_mag = models.FloatField(blank=True, null=True)
    earliest_filter = models.CharField(max_length=80, blank=True)
    ra_avg = models.FloatField(blank=True, null=True)
    dec_avg = models.FloatField(blank=True, null=True)
    catalogue = models.CharField(max_length=60, blank=True)
    catalogue_object_id = models.CharField(max_length=30, blank=True)
    separation = models.FloatField(blank=True, null=True)
    external_crossmatches = models.CharField(max_length=330, blank=True)
    discovery_target = models.CharField(max_length=80, blank=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_latest_object_stats'

    @property
    def ra_sex(self):
        """ra_sex.
        """
        ra_in_sex = ra_to_sex (self.ra_avg)
        return ra_in_sex

    @property
    def dec_sex(self):
        """dec_sex.
        """
        dec_in_sex = dec_to_sex (self.dec_avg)
        return dec_in_sex

    @property
    def earliest_mjd_date(self):
        """earliest_mjd_date.
        """
        dateFraction = getDateFractionMJD(self.earliest_mjd, delimiter = '-')
        return dateFraction

    @property
    def latest_mjd_date(self):
        """latest_mjd_date.
        """
        dateFraction = getDateFractionMJD(self.latest_mjd, delimiter = '-')
        return dateFraction


class TcsObjectGroupDefinitions(models.Model):
    """TcsObjectGroupDefinitions.
    """

    id = models.IntegerField(primary_key=True)  # AutoField?
    name = models.CharField(max_length=20)
    description = models.CharField(max_length=80, blank=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_object_group_definitions'


class TcsObjectGroups(models.Model):
    """TcsObjectGroups.
    """

    id = models.BigIntegerField(primary_key=True)
    transient_object_id = models.ForeignKey(AtlasDiffObjects, to_field='id', db_column='transient_object_id', on_delete=models.CASCADE)
    object_group_id = models.IntegerField()

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_object_groups'


class TcsParameterDefinitions(models.Model):
    """TcsParameterDefinitions.
    """

    id = models.IntegerField(primary_key=True)  # AutoField?
    name = models.CharField(max_length=40)
    type = models.CharField(max_length=20)
    value = models.CharField(max_length=40)
    units = models.CharField(max_length=40)
    catalogue_table_id = models.IntegerField()
    search_parameters_id = models.IntegerField()

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_parameter_definitions'


class TcsPostageStampImages(models.Model):
    """TcsPostageStampImages.
    """

    id = models.BigIntegerField(primary_key=True)
    image_type = models.CharField(max_length=20)
    image_filename = models.CharField(max_length=255)
    pss_filename = models.CharField(max_length=255, blank=True)
    mjd_obs = models.FloatField(blank=True, null=True)
    image_group_id = models.ForeignKey(TcsImageGroups, to_field='id', db_column='image_group_id', on_delete=models.CASCADE)
    pss_error_code = models.IntegerField()
    filter = models.CharField(max_length=80, blank=True)
    mask_ratio = models.FloatField(blank=True, null=True)
    mask_ratio_at_core = models.FloatField(blank=True, null=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_postage_stamp_images'


    @property
    def filename(self):
        """filename.
        """
        mjdstring = self.image_filename.split('_')[2][3:8]
        f = mjdstring + '/' + self.image_filename
        return f

    @property
    def stamp_mjd(self):
        """stamp_mjd.
        """
        return self.mjd_obs

    @property
    def whole_mjd(self):
        """whole_mjd.
        """
        # 2022-01-01 KWS Use the exposure name buried in the 3rd field and take the MJD which is
        #                always characters 3 to 7. DO NOT use MJD - since in S. Africa, the int MJD
        #                at the start of the night is always the previous day.
        if self.image_type == 'reffinder':
            m = int(self.mjd_obs)
        else:
            fields = self.image_filename.split('_')
            m = int(fields[2][3:8])
        return m

class TcsPostageStampRequests(models.Model):
    """TcsPostageStampRequests.
    """

    id = models.BigIntegerField(primary_key=True)
    name = models.CharField(unique=True, max_length=80)
    pss_id = models.BigIntegerField(blank=True, null=True)
    download_attempts = models.IntegerField()
    status = models.IntegerField()
    created = models.DateTimeField()
    updated = models.DateTimeField(blank=True, null=True)
    request_type = models.IntegerField(blank=True, null=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_postage_stamp_requests'


class TcsPostageStampStatusCodes(models.Model):
    """TcsPostageStampStatusCodes.
    """

    id = models.IntegerField(primary_key=True)  # AutoField?
    code = models.CharField(max_length=20)
    description = models.CharField(max_length=80, blank=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_postage_stamp_status_codes'


class TcsProcessingStatus(models.Model):
    """TcsProcessingStatus.
    """

    id = models.BigIntegerField(primary_key=True)
    status = models.IntegerField()
    started = models.DateTimeField()
    modified = models.DateTimeField(blank=True, null=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_processing_status'


class TcsSearchParameters(models.Model):
    """TcsSearchParameters.
    """

    id = models.IntegerField(primary_key=True)  # AutoField?
    parameter_set_name = models.CharField(max_length=20)
    comments = models.CharField(max_length=80, blank=True)
    last_updated = models.DateTimeField(blank=True, null=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_search_parameters'


class TcsZooRequests(models.Model):
    """TcsZooRequests.
    """

    id = models.BigIntegerField(primary_key=True)
    name = models.CharField(unique=True, max_length=80)
    zoo_id = models.BigIntegerField(blank=True, null=True)
    download_attempts = models.IntegerField()
    status = models.IntegerField()
    created = models.DateTimeField()
    updated = models.DateTimeField(blank=True, null=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_zoo_requests'

# 2017-03-21 KWS Four new database tables for Sherlock and Gravitational Waves.
# 2017-06-20 KWS Dave has updated the definition of sherlock_classifications.
class SherlockClassifications(models.Model):
    """SherlockClassifications.
    """

    transient_object_id = models.OneToOneField(AtlasDiffObjects, to_field='id', db_column='transient_object_id', primary_key=True, on_delete=models.CASCADE)
    classification = models.CharField(max_length=45, blank=True, null=True)
    annotation = models.TextField(blank=True, null=True)
    summary = models.CharField(max_length=50, blank=True, null=True)
    matchverified = models.IntegerField(db_column='matchVerified', blank=True, null=True)  # Field name made lowercase.
    developmentcomment = models.CharField(db_column='developmentComment', max_length=100, blank=True, null=True)  # Field name made lowercase.
    datelastmodified = models.DateTimeField(db_column='dateLastModified', blank=True, null=True)  # Field name made lowercase.
    datecreated = models.DateTimeField(db_column='dateCreated', blank=True, null=True)  # Field name made lowercase.
    updated = models.CharField(max_length=45, blank=True, null=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'sherlock_classifications'


# 2017-06-20 KWS Dave has updated the definition of sherlock_crossmatches.
class SherlockCrossmatches(models.Model):
    """SherlockCrossmatches.
    """

    transient_object_id = models.ForeignKey(AtlasDiffObjects, to_field='id', db_column='transient_object_id', on_delete=models.CASCADE)
    catalogue_object_id = models.CharField(max_length=30, blank=True, null=True)
    catalogue_table_id = models.SmallIntegerField(blank=True, null=True)
    separationarcsec = models.FloatField(db_column='separationArcsec', blank=True, null=True)  # Field name made lowercase.
    northseparationarcsec = models.FloatField(db_column='northSeparationArcsec', blank=True, null=True)  # Field name made lowercase.
    eastseparationarcsec = models.FloatField(db_column='eastSeparationArcsec', blank=True, null=True)  # Field name made lowercase.
    id = models.BigIntegerField(primary_key=True)
    z = models.FloatField(blank=True, null=True)
    scale = models.FloatField(blank=True, null=True)
    distance = models.FloatField(blank=True, null=True)
    distance_modulus = models.FloatField(blank=True, null=True)
    photoz = models.FloatField(db_column='photoZ', blank=True, null=True)  # Field name made lowercase.
    photozerr = models.FloatField(db_column='photoZErr', blank=True, null=True)  # Field name made lowercase.
    association_type = models.CharField(max_length=45, blank=True, null=True)
    datecreated = models.DateTimeField(db_column='dateCreated', blank=True, null=True)  # Field name made lowercase.
    physical_separation_kpc = models.FloatField(blank=True, null=True)
    catalogue_object_type = models.CharField(max_length=45, blank=True, null=True)
    catalogue_object_subtype = models.CharField(max_length=45, blank=True, null=True)
    association_rank = models.IntegerField(blank=True, null=True)
    catalogue_table_name = models.CharField(max_length=100, blank=True, null=True)
    catalogue_view_name = models.CharField(max_length=100, blank=True, null=True)
    rank = models.IntegerField(blank=True, null=True)
    rankscore = models.FloatField(db_column='rankScore', blank=True, null=True)  # Field name made lowercase.
    search_name = models.CharField(max_length=100, blank=True, null=True)
    major_axis_arcsec = models.FloatField(blank=True, null=True)
    direct_distance = models.FloatField(blank=True, null=True)
    direct_distance_scale = models.FloatField(blank=True, null=True)
    direct_distance_modulus = models.FloatField(blank=True, null=True)
    radeg = models.FloatField(db_column='raDeg', blank=True, null=True)  # Field name made lowercase.
    decdeg = models.FloatField(db_column='decDeg', blank=True, null=True)  # Field name made lowercase.
    original_search_radius_arcsec = models.FloatField(blank=True, null=True)
    catalogue_view_id = models.IntegerField(blank=True, null=True)
    u = models.FloatField(db_column='U', blank=True, null=True)  # Field name made lowercase.
    uerr = models.FloatField(db_column='UErr', blank=True, null=True)  # Field name made lowercase.
    b = models.FloatField(db_column='B', blank=True, null=True)  # Field name made lowercase.
    berr = models.FloatField(db_column='BErr', blank=True, null=True)  # Field name made lowercase.
    v = models.FloatField(db_column='V', blank=True, null=True)  # Field name made lowercase.
    verr = models.FloatField(db_column='VErr', blank=True, null=True)  # Field name made lowercase.
    r = models.FloatField(db_column='R', blank=True, null=True)  # Field name made lowercase.
    rerr = models.FloatField(db_column='RErr', blank=True, null=True)  # Field name made lowercase.
    i = models.FloatField(db_column='I', blank=True, null=True)  # Field name made lowercase.
    ierr = models.FloatField(db_column='IErr', blank=True, null=True)  # Field name made lowercase.
    j = models.FloatField(db_column='J', blank=True, null=True)  # Field name made lowercase.
    jerr = models.FloatField(db_column='JErr', blank=True, null=True)  # Field name made lowercase.
    h = models.FloatField(db_column='H', blank=True, null=True)  # Field name made lowercase.
    herr = models.FloatField(db_column='HErr', blank=True, null=True)  # Field name made lowercase.
    k = models.FloatField(db_column='K', blank=True, null=True)  # Field name made lowercase.
    kerr = models.FloatField(db_column='KErr', blank=True, null=True)  # Field name made lowercase.
    field_u = models.FloatField(db_column='_u', blank=True, null=True)  # Field renamed because it started with '_'.
    field_uerr = models.FloatField(db_column='_uErr', blank=True, null=True)  # Field name made lowercase. Field renamed because it started with '_'.
    field_g = models.FloatField(db_column='_g', blank=True, null=True)  # Field renamed because it started with '_'.
    field_gerr = models.FloatField(db_column='_gErr', blank=True, null=True)  # Field name made lowercase. Field renamed because it started with '_'.
    field_r = models.FloatField(db_column='_r', blank=True, null=True)  # Field renamed because it started with '_'.
    field_rerr = models.FloatField(db_column='_rErr', blank=True, null=True)  # Field name made lowercase. Field renamed because it started with '_'.
    field_i = models.FloatField(db_column='_i', blank=True, null=True)  # Field renamed because it started with '_'.
    field_ierr = models.FloatField(db_column='_iErr', blank=True, null=True)  # Field name made lowercase. Field renamed because it started with '_'.
    field_z = models.FloatField(db_column='_z', blank=True, null=True)  # Field renamed because it started with '_'.
    field_zerr = models.FloatField(db_column='_zErr', blank=True, null=True)  # Field name made lowercase. Field renamed because it started with '_'.
    field_y = models.FloatField(db_column='_y', blank=True, null=True)  # Field renamed because it started with '_'.
    field_yerr = models.FloatField(db_column='_yErr', blank=True, null=True)  # Field name made lowercase. Field renamed because it started with '_'.
    g = models.FloatField(db_column='G', blank=True, null=True)  # Field name made lowercase.
    gerr = models.FloatField(db_column='GErr', blank=True, null=True)  # Field name made lowercase.
    unkmag = models.FloatField(db_column='unkMag', blank=True, null=True)  # Field name made lowercase.
    unkmagerr = models.FloatField(db_column='unkMagErr', blank=True, null=True)  # Field name made lowercase.
    datelastmodified = models.DateTimeField(db_column='dateLastModified', blank=True, null=True)  # Field name made lowercase.
    updated = models.IntegerField(blank=True, null=True)
    classificationreliability = models.IntegerField(db_column='classificationReliability', blank=True, null=True)  # Field name made lowercase.
    transientabsmag = models.FloatField(db_column='transientAbsMag', blank=True, null=True)  # Field name made lowercase.

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'sherlock_crossmatches'


class TcsGravityEvents(models.Model):
    """TcsGravityEvents.
    """

    gravity_event_id = models.CharField(primary_key=True, max_length=10)
    gracedb_id = models.CharField(max_length=10)
    mjd = models.FloatField(blank=True, null=True)
    datelastmodified = models.DateTimeField(db_column='dateLastModified', blank=True, null=True)  # Field name made lowercase.
    updated = models.IntegerField(blank=True, null=True)

    class Meta:
        """Meta.
        """

        db_table = 'tcs_gravity_events'


# 2023-06-15 KWS Added "interesting" column to human override all other significance factors.
class TcsGravityAlerts(models.Model):
    id = models.BigIntegerField(primary_key=True)
    superevent_id = models.CharField(max_length=20)
    significant = models.IntegerField(blank=True, null=True)
    alert_type = models.CharField(max_length=20, blank=True, null=True)
    alert_time = models.DateTimeField(blank=True, null=True)
    mjd_obs = models.FloatField(blank=True, null=True)
    far = models.FloatField(blank=True, null=True)
    distmean = models.FloatField(blank=True, null=True)
    diststd = models.FloatField(blank=True, null=True)
    class_bbh = models.FloatField(blank=True, null=True)
    class_bns = models.FloatField(blank=True, null=True)
    class_nsbh = models.FloatField(blank=True, null=True)
    class_terrestrial = models.FloatField(blank=True, null=True)
    prop_hasns = models.FloatField(blank=True, null=True)
    prop_hasremnant = models.FloatField(blank=True, null=True)
    prop_hasmassgap = models.FloatField(blank=True, null=True)
    area10 = models.FloatField(blank=True, null=True)
    area50 = models.FloatField(blank=True, null=True)
    area90 = models.FloatField(blank=True, null=True)
    creator = models.CharField(max_length=30, blank=True, null=True)
    group = models.CharField(max_length=100, blank=True, null=True)
    pipeline = models.CharField(max_length=100, blank=True, null=True)
    map_iteration = models.CharField(max_length=100, blank=True, null=True)
    alert_map = models.CharField(db_column='map', max_length=400, blank=True, null=True)
    interesting = models.IntegerField(blank=True, null=True)
    dateadded = models.DateTimeField(db_column='dateAdded', blank=True, null=True)  # Field name made lowercase.
    datelastmodified = models.DateTimeField(db_column='dateLastModified', blank=True, null=True)  # Field name made lowercase.

    @property
    def class_bbh_pc(self):
        """class_bbh_pc.
        """
        p = None
        if self.class_bbh is not None:
            p = self.class_bbh * 100.0
        return p

    @property
    def class_bns_pc(self):
        """class_bns_pc.
        """
        p = None
        if self.class_bns is not None:
            p = self.class_bns * 100.0
        return p

    @property
    def class_nsbh_pc(self):
        """class_nsbh_pc.
        """
        p = None
        if self.class_nsbh is not None:
            p = self.class_nsbh * 100.0
        return p

    @property
    def class_terrestrial_pc(self):
        """class_terrestrial_pc.
        """
        p = None
        if self.class_terrestrial is not None:
            p = self.class_terrestrial * 100.0
        return p

    class Meta:
        managed = False
        db_table = 'tcs_gravity_alerts'
        unique_together = (('superevent_id', 'alert_time', 'alert_type'),)


#class TcsGravityEventAnnotations(models.Model):
#    """TcsGravityEventAnnotations.
#    """
#
#    primaryid = models.BigIntegerField(db_column='primaryId', primary_key=True)  # Field name made lowercase.
#    transient_object_id = models.ForeignKey(AtlasDiffObjects, to_field='id', db_column='transient_object_id', on_delete=models.CASCADE)
#    gravity_event_id = models.ForeignKey(TcsGravityEvents, to_field='gravity_event_id', db_column='gravity_event_id', on_delete=models.CASCADE)
#    gracedb_id = models.CharField(max_length=10)
#    enclosing_contour = models.IntegerField(blank=True, null=True)
#    map_name = models.CharField(max_length=30, blank=True, null=True)
#    datelastmodified = models.DateTimeField(db_column='dateLastModified', blank=True, null=True)  # Field name made lowercase.
#    updated = models.IntegerField(blank=True, null=True)
#
#    class Meta:
#        """Meta.
#        """
#
#        db_table = 'tcs_gravity_event_annotations'
#        unique_together = (('transient_object_id', 'gracedb_id'),)

# 2023-05-24 KWS Added new columns to TcsGravityEventAnnotations

class TcsGravityEventAnnotations(models.Model):
    primaryid = models.BigIntegerField(db_column='primaryId', primary_key=True)  # Field name made lowercase.
    transient_object_id = models.ForeignKey(AtlasDiffObjects, to_field='id', db_column='transient_object_id', on_delete=models.CASCADE)
    gravity_event_id = models.CharField(max_length=10, db_collation='utf8_swedish_ci')
    gracedb_id = models.CharField(max_length=10, db_collation='utf8_swedish_ci')
    enclosing_contour = models.IntegerField(blank=True, null=True)
    map_name = models.CharField(max_length=100, blank=True, null=True)
    #map_iteration = models.CharField(max_length=100, blank=True, null=True)
    map_iteration = models.ForeignKey('TcsGravityAlerts', to_field='map_iteration', db_column='map_iteration', on_delete=models.CASCADE)
    days_since_event = models.FloatField(blank=True, null=True)
    probability = models.FloatField(blank=True, null=True)
    distance = models.FloatField(blank=True, null=True)
    distance_sigma = models.FloatField(blank=True, null=True)
    datelastmodified = models.DateTimeField(db_column='dateLastModified', blank=True, null=True)  # Field name made lowercase.
    updated = models.IntegerField(blank=True, null=True)
    datecreated = models.DateTimeField(db_column='dateCreated', blank=True, null=True)  # Field name made lowercase.

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_gravity_event_annotations'
        unique_together = (('transient_object_id', 'gravity_event_id', 'map_iteration'),)


# 2017-05-09 KWS Added TcsObjectComments table. Comments will be inserted here in future.
class TcsObjectComments(models.Model):
    """TcsObjectComments.
    """

    id = models.BigIntegerField(db_column='id', primary_key=True)
    transient_object_id = models.ForeignKey(AtlasDiffObjects, to_field='id', db_column='transient_object_id', on_delete=models.CASCADE)
    date_inserted = models.DateTimeField(db_column='date_inserted', blank=False, null=False)
    comment = models.CharField(max_length=768, db_column='comment', blank=True, null=True)
    username = models.CharField(max_length=90, db_column='username', blank=True, null=True)

    @property
    def date_inserted_mjd(self):
        """date_inserted_mjd.
        """
        date_inserted_mjd = getMJDFromSqlDate(self.date_inserted.strftime("%Y-%m-%d %H:%M:%S"))
        return date_inserted_mjd


    class Meta:
        """Meta.
        """

        db_table = 'tcs_object_comments'


# 2019-06-06 KWS Added AtlasStackedForcedPhotometry table.
class AtlasStackedForcedPhotometry(models.Model):
    """AtlasStackedForcedPhotometry.
    """

    id = models.BigIntegerField(primary_key=True)
    atlas_object_id = models.ForeignKey(AtlasDiffObjects, to_field='id', db_column='atlas_object_id', on_delete=models.CASCADE)
    mjd = models.FloatField(blank=True, null=True)
    m = models.FloatField(blank=True, null=True)
    dm = models.FloatField(blank=True, null=True)
    ujy = models.FloatField(blank=True, null=True)
    dujy = models.FloatField(blank=True, null=True)
    f = models.CharField(max_length=10, blank=True, null=True)
    err = models.FloatField(blank=True, null=True)
    chin = models.FloatField(blank=True, null=True)
    ra = models.FloatField(blank=True, null=True)
    dec = models.FloatField(blank=True, null=True)
    x = models.FloatField(blank=True, null=True)
    y = models.FloatField(blank=True, null=True)
    maj = models.FloatField(blank=True, null=True)
    min = models.FloatField(blank=True, null=True)
    phi = models.FloatField(blank=True, null=True)
    apfit = models.FloatField(blank=True, null=True)
    sky = models.FloatField(blank=True, null=True)
    zp = models.FloatField(blank=True, null=True)
    stack = models.CharField(max_length=60, blank=True, null=True)
    date_inserted = models.DateTimeField()
    redo_photometry = models.IntegerField(blank=True, null=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'atlas_stacked_forced_photometry'

# 2022-05-06 KWS Added AtlasDiffSubcells for heatmap plots.

class AtlasDiffSubcells(models.Model):
    id = models.BigAutoField(primary_key=True)
    obs = models.CharField(max_length=60, blank=True, null=True)
    region = models.PositiveSmallIntegerField()
    ndet = models.PositiveIntegerField()

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'atlas_diff_subcells'
        unique_together = (('obs', 'region'),)


class AtlasHeatmaps(models.Model):
    id = models.BigAutoField(primary_key=True)
    site = models.CharField(max_length=10, blank=True, null=True)
    region = models.PositiveIntegerField()
    ndet = models.PositiveIntegerField()

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'atlas_heatmaps'
        unique_together = (('site', 'region'),)


# 2024-01-17 KWS Added new tcs_vra_scores table for the Virtual Research Assistant.
class TcsVraScores(models.Model):
    id = models.BigAutoField(primary_key=True)
    transient_object_id = models.ForeignKey(AtlasDiffObjects, to_field='id', db_column='transient_object_id', on_delete=models.CASCADE)
    preal = models.FloatField(blank=True, null=True)
    pgal = models.FloatField(blank=True, null=True)
    pfast = models.FloatField(blank=True, null=True)
    rank = models.FloatField(blank=True, null=True)
    rank_alt1 = models.FloatField(blank=True, null=True)
    rank_alt2 = models.FloatField(blank=True, null=True)
    timestamp = models.DateTimeField()
    apiusername = models.CharField(max_length=30, blank=True, null=True)
    username = models.CharField(max_length=30, blank=True, null=True)
    debug = models.BooleanField(null=True, blank=True)

    class Meta:
        managed = False
        db_table = 'tcs_vra_scores'


# 2024-04-16 KWS Added new tcs_vra_scores table for the Virtual Research Assistant.
class TcsVraTodo(models.Model):
    transient_object_id = models.ForeignKey(AtlasDiffObjects, to_field='id', db_column='transient_object_id', on_delete=models.CASCADE, primary_key=True)
    timestamp = models.DateTimeField()

    class Meta:
        managed = False
        db_table = 'tcs_vra_todo'


# 2024-04-16 KWS Added new tcs_vra_scores table for the Virtual Research Assistant.
class TcsVraRank(models.Model):
    transient_object_id = models.ForeignKey(AtlasDiffObjects, to_field='id', db_column='transient_object_id', on_delete=models.CASCADE, primary_key=True)
    rank = models.FloatField(blank=True, null=True)
    rank_alt1 = models.FloatField(blank=True, null=True)
    rank_alt2 = models.FloatField(blank=True, null=True)
    timestamp = models.DateTimeField()

    class Meta:
        managed = False
        db_table = 'tcs_vra_rank'




import sys

from django.shortcuts import get_object_or_404, render
from rest_framework.response import Response
from rest_framework.views import APIView
from rest_framework import status
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
# 2024-01-29 KWS Need the model to do inserts.
from django.core.exceptions import ObjectDoesNotExist

from atlas.models import TcsObjectGroups
from atlas.models import TcsVraScores
from .serializers import ConeSerializer, ObjectsSerializer, ObjectListSerializer, VRAScoresSerializer, VRAScoresListSerializer, VRATodoSerializer, VRATodoListSerializer, TcsObjectGroupsSerializer, TcsObjectGroupsDeleteSerializer, TcsObjectGroupsListSerializer, VRARankSerializer, VRARankListSerializer, ExternalCrossmatchesListSerializer, ObjectDetectionListSerializer
from .query_auth import QueryAuthentication
from .permissions import IsApprovedUser

def retcode(message):
    if 'error' in message: return status.HTTP_400_BAD_REQUEST
    else:                  return status.HTTP_200_OK

class ConeView(APIView):
    authentication_classes = [TokenAuthentication, QueryAuthentication]
    permission_classes = [IsAuthenticated&IsApprovedUser]

    def get(self, request):
        serializer = ConeSerializer(data=request.GET, context={'request': request})
        if serializer.is_valid():
            message = serializer.save()
            return Response(message, status=retcode(message))
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    def post(self, request, format=None):
        serializer = ConeSerializer(data=request.data, context={'request': request})
        if serializer.is_valid():
            message = serializer.save()
            return Response(message, status=retcode(message))
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class ObjectsView(APIView):
    authentication_classes = [TokenAuthentication, QueryAuthentication]
    permission_classes = [IsAuthenticated&IsApprovedUser]

    def get(self, request):
        serializer = ObjectsSerializer(data=request.GET, context={'request': request})
        if serializer.is_valid():
            message = serializer.save()
            return Response(message, status=retcode(message))
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    def post(self, request, format=None):
        serializer = ObjectsSerializer(data=request.data, context={'request': request})
        if serializer.is_valid():
            message = serializer.save()
            return Response(message, status=retcode(message))
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class ObjectListView(APIView):
    authentication_classes = [TokenAuthentication, QueryAuthentication]
    permission_classes = [IsAuthenticated&IsApprovedUser]

    def get(self, request):
        serializer = ObjectListSerializer(data=request.GET, context={'request': request})
        if serializer.is_valid():
            message = serializer.save()
            return Response(message, status=retcode(message))
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    def post(self, request, format=None):
        serializer = ObjectListSerializer(data=request.data, context={'request': request})
        if serializer.is_valid():
            message = serializer.save()
            return Response(message, status=retcode(message))
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class VRAScoresView(APIView):
    authentication_classes = [TokenAuthentication, QueryAuthentication]
    permission_classes = [IsAuthenticated&IsApprovedUser]

    def get(self, request):
        return Response({"Error": "GET is not implemented for this service."})

    def post(self, request, format=None):
        serializer = VRAScoresSerializer(data=request.data, context={'request': request})

        if serializer.is_valid():
            message = serializer.save()

            return Response(message, status=status.HTTP_201_CREATED)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class VRAScoresListView(APIView):
    authentication_classes = [TokenAuthentication, QueryAuthentication]
    permission_classes = [IsAuthenticated&IsApprovedUser]

    def get(self, request):
        serializer = VRAScoresListSerializer(data=request.GET, context={'request': request})
        if serializer.is_valid():
            message = serializer.save()
            return Response(message, status=retcode(message))
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    def post(self, request, format=None):
        serializer = VRAScoresListSerializer(data=request.data, context={'request': request})
        if serializer.is_valid():
            message = serializer.save()
            return Response(message, status=retcode(message))
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

# 2024-04-16 KWS Added VRATodoView. NOTE: Add code to read the reply message and generate a sensible HTTP response
#                appropriate to the circumstances. E.g. if object is not found generate a 404, etc.
class VRATodoView(APIView):
    authentication_classes = [TokenAuthentication, QueryAuthentication]
    permission_classes = [IsAuthenticated&IsApprovedUser]

    def get(self, request):
        return Response({"Error": "GET is not implemented for this service."})

    def post(self, request, format=None):
        serializer = VRATodoSerializer(data=request.data, context={'request': request})
    
        if serializer.is_valid():
            message = serializer.save()
    
            return Response(message, status=status.HTTP_201_CREATED)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
            
# 2024-05-07 KWS Added VRATodoListView.
class VRATodoListView(APIView):
    authentication_classes = [TokenAuthentication, QueryAuthentication]
    permission_classes = [IsAuthenticated&IsApprovedUser]

    def get(self, request):
        serializer = VRATodoListSerializer(data=request.GET, context={'request': request})
        if serializer.is_valid():
            message = serializer.save()
            return Response(message, status=retcode(message))
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    def post(self, request, format=None):
        serializer = VRATodoListSerializer(data=request.data, context={'request': request})
        if serializer.is_valid():
            message = serializer.save()
            return Response(message, status=retcode(message))
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

class TcsObjectGroupsView(APIView):
    authentication_classes = [TokenAuthentication, QueryAuthentication]
    permission_classes = [IsAuthenticated&IsApprovedUser]

    def get(self, request):
        return Response({"Error": "GET is not implemented for this service."})

    def post(self, request, format=None):
        serializer = TcsObjectGroupsSerializer(data=request.data, context={'request': request})
    
        if serializer.is_valid():
            message = serializer.save()
    
            return Response(message, status=status.HTTP_201_CREATED)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

class TcsObjectGroupsListView(APIView):
    authentication_classes = [TokenAuthentication, QueryAuthentication]
    permission_classes = [IsAuthenticated&IsApprovedUser]

    def get(self, request):
        serializer = TcsObjectGroupsListSerializer(data=request.GET, context={'request': request})
        if serializer.is_valid():
            message = serializer.save()
            return Response(message, status=retcode(message))
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    def post(self, request, format=None):
        serializer = TcsObjectGroupsListSerializer(data=request.data, context={'request': request})
        if serializer.is_valid():
            message = serializer.save()
            return Response(message, status=retcode(message))
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class TcsObjectGroupsDeleteView(APIView):
    authentication_classes = [TokenAuthentication, QueryAuthentication]
    permission_classes = [IsAuthenticated&IsApprovedUser]

    def get(self, request):
        return Response({"Error": "GET is not implemented for this service."})

    def post(self, request, format=None):
        serializer = TcsObjectGroupsDeleteSerializer(data=request.data, context={'request': request})
    
        if serializer.is_valid():
            message = serializer.save()
    
        if "deleted" in message['info']:
            # No point returning the message info. 204s will drop the message content anyway.
            return Response(status=status.HTTP_204_NO_CONTENT)
        elif "does not exist" in message['info']:
            return Response(message, status=status.HTTP_400_BAD_REQUEST)
        else:
            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

# 2024-05-22 KWS Added VRARankView. NOTE: Add code to read the reply message and generate a sensible HTTP response
#                appropriate to the circumstances. E.g. if object is not found generate a 404, etc.
class VRARankView(APIView):
    authentication_classes = [TokenAuthentication, QueryAuthentication]
    permission_classes = [IsAuthenticated&IsApprovedUser]

    def get(self, request):
        return Response({"Error": "GET is not implemented for this service."})

    def post(self, request, format=None):
        serializer = VRARankSerializer(data=request.data, context={'request': request})
    
        if serializer.is_valid():
            message = serializer.save()
    
            return Response(message, status=status.HTTP_201_CREATED)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
            
# 2024-05-22 KWS Added VRARankListView.
class VRARankListView(APIView):
    authentication_classes = [TokenAuthentication, QueryAuthentication]
    permission_classes = [IsAuthenticated&IsApprovedUser]

    def get(self, request):
        serializer = VRARankListSerializer(data=request.GET, context={'request': request})
        if serializer.is_valid():
            message = serializer.save()
            return Response(message, status=retcode(message))
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    def post(self, request, format=None):
        serializer = VRARankListSerializer(data=request.data, context={'request': request})
        if serializer.is_valid():
            message = serializer.save()
            return Response(message, status=retcode(message))
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


# 2024-09-24 KWS Added ExternalCrossmatchesListView.
class ExternalCrossmatchesListView(APIView):
    authentication_classes = [TokenAuthentication, QueryAuthentication]
    permission_classes = [IsAuthenticated&IsApprovedUser]

    def get(self, request):
        serializer = ExternalCrossmatchesListSerializer(data=request.GET, context={'request': request})
        if serializer.is_valid():
            message = serializer.save()
            return Response(message, status=retcode(message))
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    def post(self, request, format=None):
        serializer = ExternalCrossmatchesListSerializer(data=request.data, context={'request': request})
        if serializer.is_valid():
            message = serializer.save()
            return Response(message, status=retcode(message))
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


# 2024-09-24 KWS Added ExternalCrossmatchesListView.
class ObjectDetectionListView(APIView):
    authentication_classes = [TokenAuthentication, QueryAuthentication]
    permission_classes = [IsAuthenticated&IsApprovedUser]

    def get(self, request):
        serializer = ObjectDetectionListSerializer(data=request.GET, context={'request': request})
        if serializer.is_valid():
            message = serializer.save()
            return Response(message, status=retcode(message))
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    def post(self, request, format=None):
        serializer = ObjectDetectionListSerializer(data=request.data, context={'request': request})
        if serializer.is_valid():
            message = serializer.save()
            return Response(message, status=retcode(message))
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

